<?php

namespace WilokeListgoFunctionality\Controllers;


use WilokeListgoFunctionality\Framework\Helpers\GetSettings;
use WilokeListgoFunctionality\Framework\Helpers\SetSettings;
use WilokeListgoFunctionality\Framework\Payment\PaymentConfiguration;
use WilokeListgoFunctionality\Model\PostsModel;

class ExpirationPosts extends Controller {
	private $listingExpiryKey = 'listgo_listing_expiry';
	private $listingAlmostExpiryKey = 'listgo_listing_almost_expiry';

	public function __construct() {
		add_action('updated_post_meta', array($this, 'changePremiumListingSettings'), 10, 4);
		add_action('added_post_meta', array($this, 'addPremiumListingSettings'), 10, 4);
		add_action('listgo_listing_expiry', array($this, 'moveExpiryEventsToTrash'));
		add_action('wiloke/listgo/after-changed-post-status', array($this, 'updateExpiryDate'), 10, 3);
		// add_action('wiloke/listgo/after-changed-post-status', array($this, 'updateListingPlans'), 10, 3);
		add_action('wiloke/listgo/after-migrated-post-to-expired-box', array($this, 'clearExpirySchedules'));

		add_action('wiloke/listgo/after-updated-plan-id', array($this, 'updatePlanID'), 10, 2);

		add_action('save_post_listing', array($this, 'updateExpirationDate'), 10, 2);
		add_action('wiloke-submission/setup-free-plan', array($this, 'updatePremiumPlanSettings'), 5);
		add_action('post_updated', array($this, 'unScheduleSingleEventAfterMovingListingToTrash'), 10, 3);
	}

	public function unScheduleSingleEventAfterMovingListingToTrash($postID, $oAfter, $oBefore){
		if ( ($oBefore->post_status == $oAfter->post_status) || ($oAfter->post_status == 'publish') ) {
			return false;
		}

		if ( $oAfter->post_status  != 'draft' && $oAfter->post_status  != 'trash' ){
			return false;
		}

		$postType = get_post_type($postID);

		if ( $postType != 'listing' ){
			return false;
		}

		wp_clear_scheduled_hook($this->listingExpiryKey, array($postID));
		wp_clear_scheduled_hook($this->listingExpiryKey, array("$postID"));
		wp_clear_scheduled_hook($this->listingAlmostExpiryKey, array($postID));
		wp_clear_scheduled_hook($this->listingAlmostExpiryKey, array("$postID"));
	}

	private function getPremiumKey($postType){
		return $postType == 'listing' ? 'listing_premium' : 'event_premium';
	}

	public function updatePremiumPlanSettings($aResponse){
		$aPlanSettings = GetSettings::getPostMeta($aResponse['planID']);
		$duration = $aPlanSettings['duration'];
		
		if ( !empty($duration) ){
			$aData['expiry_date'] = strtotime($post->post_date . ' +'.$duration . ' day');
		}
		$aData['plan'] = $aResponse['planID'];
		
		SetSettings::setPostMeta($aResponse['listingID'], wilokeRepository('app:belongsToPlanID'), $aData['plan']);
		SetSettings::setPostMeta($aResponse['listingID'], 'listing_premium', $aData);
	}

	public function updateExpirationDate($postID, $post){
		if ( $post->post_type !== 'listing' ){
			return false;
		}

		$aData = GetSettings::getPostMeta($postID, 'listing_premium');

		if ( empty($aData) ){
			return false;
		}

		if ( isset($aData['expiry_date']) && is_numeric($aData['expiry_date'])  ){
			return false;
		}

		if ( !isset($aData['plan']) || empty($aData['plan']) ){
			return false;
		}

		$aPlanSettings = GetSettings::getPostMeta($aData['plan']);

		if ( PaymentConfiguration::isNonRecurringPayment() ){
			$duration = $aPlanSettings['duration'];
		}else{
			$duration = $aPlanSettings['regular_period'];
		}

		if ( empty($duration) ){
			return false;
		}

		$aData['expiry_date'] = strtotime($post->post_date . ' +'.$duration . ' day');

		SetSettings::setPostMeta($postID, 'listing_premium', $aData);
	}

	public function updatePlanID($postID, $planID){
		$postType = get_post_type($postID);
		$postMetaKey = $postType == 'listing' ? 'listing_premium' : 'event_premium';
		// $aData = GetSettings::getPostMeta($postID, $postMetaKey);
		$aData['plan'] = $planID;
		
		$aPlanSettings = GetSettings::getPostMeta($planID);

		if ( !empty($aPlanSettings['duration']) ){
			$aCurrentPlan = GetSettings::getPostMeta($postID, $postMetaKey);
			$nextBillingDate = is_numeric($aCurrentPlan['nextBillingDate']) ? $aCurrentPlan['nextBillingDate'] : strtotime($aCurrentPlan['nextBillingDate']);
			$aData['expiry_date'] = empty($nextBillingDate) ? strtotime('+'.$aPlanSettings['duration'] . ' day', current_time('timestamp'))  :  strtotime('+'.$aPlanSettings['duration'] . ' day', $nextBillingDate);
		}

		// wp_send_json_error(array(
		// 	'msg' => 'x',
		// 	'planSettings' => $aPlanSettings,
		// 	'aData' => $aData,
		// 	'postID' => $postID,
		// 	'postKey' => $postMetaKey
		// ));

		SetSettings::setPostMeta($postID, $postMetaKey, $aData);
	}

	public function clearExpirySchedules($aPostIDs){
		if ( empty($aPostIDs) ){
			return false;
		}

		foreach ($aPostIDs as $postID){
			$postType = get_post_type($postID);

			if ( $postType != 'listing' ){
				continue;
			}

			$postMetaKey = 'listing_premium';
			wp_clear_scheduled_hook($this->listingExpiryKey, array($postID));
			wp_clear_scheduled_hook($this->listingAlmostExpiryKey, array($postID));

			$aData = GetSettings::getPostMeta($postID, $postMetaKey);
			$aData['expiry_date'] = '';
			update_post_meta($postID, $postMetaKey, $aData);
		}
	}

	public function moveExpiryEventsToTrash($postID){
		$status = get_post_type($postID) == 'event' ? 'draft' : 'expired';
		PostsModel::updatePostsStatus(array($postID), $status);

		do_action('wiloke-submission/mail', array(
			'type'      => wilokeRepository('mailstatus:type', true)->sub('postExpired'),
			'target'    => array($postID)
		));
	}

	private function setExpiryDate($postID, $expirationDate){
		$postType = get_post_type($postID);
		if ( $postType !== 'listing' ){
			return false;
		}

		wp_clear_scheduled_hook($this->listingExpiryKey, array($postID));
		wp_clear_scheduled_hook($this->listingExpiryKey, array("$postID"));
		wp_clear_scheduled_hook($this->listingAlmostExpiryKey, array($postID));
		wp_clear_scheduled_hook($this->listingAlmostExpiryKey, array("$postID"));

		$beforeOneDay = $expirationDate - 86400;

		if ( $beforeOneDay > 0 ){
			wp_schedule_single_event($beforeOneDay, $this->listingAlmostExpiryKey, array($postID));
		}else{
			$beforeHalfDay = $expirationDate - 43200;
			wp_schedule_single_event($beforeHalfDay, $this->listingAlmostExpiryKey, array($postID));
		}
		
		wp_schedule_single_event($expirationDate, $this->listingExpiryKey, array($postID));
	}

	public function addPremiumListingSettings($metaID, $postID, $metaKey, $aMetaVal){
		if ( $metaKey !== 'listing_premium' && $metaKey !== 'event_premium' ){
			return false;
		}

		if ( $metaKey == 'listing_premium' ){
			if ( isset($aMetaVal['expiry_date']) && !empty($aMetaVal['expiry_date']) ){
				$expiration = abs($aMetaVal['expiry_date']);
				$this->setExpiryDate($postID, $expiration);
			}
		}

		if ( isset($aMetaVal['plan']) && !empty($aMetaVal['plan']) ){
			SetSettings::setPostMeta($postID, wilokeRepository('app:belongsToPlanID'), $aMetaVal['plan']);
		}
	}

	public function changePremiumListingSettings($metaID, $postID, $metaKey, $aMetaVal){
		
		if ( $metaKey !== 'listing_premium' && $metaKey !== 'event_premium' ){
			return false;
		}

		if ( $metaKey == 'listing_premium' ){
			if ( isset($aMetaVal['expiry_date']) && !empty($aMetaVal['expiry_date']) ){
				$expiration = abs($aMetaVal['expiry_date']);
				$this->setExpiryDate($postID, $expiration);
			}
		}

		if ( !isset($aMetaVal['plan']) || empty($aMetaVal['plan']) ){
			delete_post_meta($postID, wilokeRepository('app:belongsToPlanID'));
		}else{
			SetSettings::setPostMeta($postID, wilokeRepository('app:belongsToPlanID'), $aMetaVal['plan']);
		}
	}

	public function updateExpiryDate($aInfo, $postID, $aPost){
		if ( get_post_type($postID) != 'listing' ){
			return false;
		}

		$nextBillingDate = is_numeric($aInfo['nextBillingDate']) ? $aInfo['nextBillingDate'] : strtotime($aInfo['nextBillingDate']);

		$this->setExpiryDate($postID, $nextBillingDate);
	}

	public function updateListingPlans($aInfo, $postID, $aPost){
		$aPlanSettings = GetSettings::getPostMeta($aResponse['planID']);
		$duration = $aPlanSettings['duration'];
		$nextBillingDate = is_numeric($aInfo['nextBillingDate']) ? $aInfo['nextBillingDate'] : strtotime($aInfo['nextBillingDate']);

		if ( !empty($duration) ){
			$aData['expiry_date'] = empty($nextBillingDate) ? strtotime('+'.$duration . ' day', current_time('timestamp'))  :  strtotime('+'.$duration . ' day', $nextBillingDate);
		}
		$aData['plan'] = $aResponse['planID'];
		
		// SetSettings::setPostMeta($aResponse['listingID'], wilokeRepository('app:belongsToPlanID'), $aData['plan']);
		SetSettings::setPostMeta($aResponse['listingID'], 'listing_premium', $aData);
	}
}