<?php
namespace WilokeListgoFunctionality\Controllers;

use WilokeListgoFunctionality\Framework\Payment\Billable;
use WilokeListgoFunctionality\Framework\Payment\Checkout;
use WilokeListgoFunctionality\Framework\Payment\DirectBankTransfer\NonRecurringPaymentMethod;
use WilokeListgoFunctionality\Framework\Payment\PaymentConfiguration;
use WilokeListgoFunctionality\Framework\Payment\PayPal\PayPalChangePlan;
use WilokeListgoFunctionality\Framework\Payment\PayPal\PayPalRecurringPaymentMethod;
use WilokeListgoFunctionality\Framework\Payment\PayPal\PaypalUpdateBillingAgreement;
use WilokeListgoFunctionality\Framework\Payment\PayPal\Webhook;
use WilokeListgoFunctionality\Framework\Payment\Receipt;
use WilokeListgoFunctionality\Framework\Payment\PayPal\PayPalNonRecurringPaymentMethod;
use WilokeListgoFunctionality\Framework\Store\Session;
use WilokeListgoFunctionality\Model\PaymentModel;
use WilokeListgoFunctionality\Model\PayPalModel;
use WilokeListgoFunctionality\Framework\Payment\PayPal\PayPalExecuteRecurringPayment;
use WilokeListgoFunctionality\Framework\Payment\PayPal\PayPalExecuteNonRecurringPayment;
use WilokeListgoFunctionality\Framework\Payment\PayPal\PayPalCancelPlan;

class PayPalController extends Controller {
	public $nonRecurringPaymentKey = 'NonRecurringPayment';
	public $recurringPaymentKey = 'RecurringPayment';
	public $gateway = 'paypal';

	public function __construct() {
		add_action('init', array($this, 'listenEvents'));
		add_action('wp_ajax_wiloke_submission_pay_with_paypal', array($this, 'preparePayment'));
		add_action('wp_ajax_nopriv_wiloke_submission_pay_with_paypal', array($this, 'preparePayment'));
		add_action('init', array($this, 'paymentExecution'), 1);
//		add_action('wp_ajax_wiloke_buy_event_plan_with_paypal', array($this, 'buyEventPlan'));
		add_action('wiloke_submission/purchase-event-plan-with-paypal', array($this, 'buyEventPlan'), 10);
		add_action('wp_ajax_listgo_cancel_plan_paypal', array($this, 'cancelSubscription'));
	}

	public function cancelSubscription(){
		$this->middleware(['isYourSession'], array(
			'sessionID' => abs($_POST['sessionID'])
		));

		$agreementID = PayPalModel::getAgreementID($_POST['sessionID']);

		if ( empty($agreementID) ){
			wp_send_json_error(esc_html__('We could not find the agreement of this session. It may the session was cancelled before.', 'wiloke'));
		}
		$oCancel = new PayPalCancelPlan($agreementID);
		$aStatus = $oCancel->cancelPlan();
		if ( $aStatus['status'] == 'success' ){
			PaymentModel::updateToCancelledStatusWhereEqualToSessionID($_POST['sessionID']);
			do_action('wiloke-submission/mail/subscription/cancelled', array(
				'customerID' => get_current_user_id(),
				'sessionID'	 => $_POST['sessionID']
			));
			wp_send_json_success(esc_html__('Congratulations! This session has been cancelled.', 'wiloke'));
		}else{
			wp_send_json_error(esc_html__('Something went wrong. We could not cancelled this plan.', 'wiloke'));
		}
	}

	public function listenEvents(){
		if ( !isset($_REQUEST['wiloke-submission-listener']) || ($_REQUEST['wiloke-submission-listener'] != $this->gateway) ){
			return false;
		}

		new Webhook();
	}

	public function updatePayPal(){
		$instPayPal = new PaypalUpdateBillingAgreement();
		$instPayPal->test($_POST['agreementID']);
	}

	public function withNonRecurringPayment(){
		$oPayPalMethod = new PayPalExecuteNonRecurringPayment();
		$oPayPalMethod->executePayment();
	}

	public function withRecurringPayment(){
		$oPayPalMethod = new PayPalExecuteRecurringPayment();
		$oPayPalMethod->executePayment();
	}

	public function paymentExecution(){
		if ( !isset($_REQUEST['billingType']) ){
			return false;
		}

		new Billable(
			array(
				'gateway' => $this->gateway,
				'planID'  => $_REQUEST['planID'],
				'isIgnoreValidatePrice'=>true
			)
		);

		if ( !PaymentConfiguration::isNonRecurringPayment($_GET['billingType']) ){
			$this->withRecurringPayment();
		}else{
			$this->withNonRecurringPayment();
		}
	}

	public function preparePayment($aData=array()){
		// Authentication
		new Billable(array(
			'gateway' => $this->gateway,
			'planID'  => Session::getSession(wilokeRepository('sessionkeys:storePlanID'))
		));
		$aData = empty($aData) ? $_POST : $aData;
		$aData['planID'] = Session::getSession(wilokeRepository('sessionkeys:storePlanID'));

		$oReceipt = new Receipt($aData);
		if ( PaymentConfiguration::isNonRecurringPayment() ){
			$oPayPalMethod = new PayPalNonRecurringPaymentMethod();
		}else{
			$oPayPalMethod = new PayPalRecurringPaymentMethod();
		}

		$oCheckout = new Checkout();
		$aCheckAcceptPaymentStatus = $oCheckout->begin($oReceipt, $oPayPalMethod);

		if ( $aCheckAcceptPaymentStatus['status'] == 'success' ){
			wp_send_json_success($aCheckAcceptPaymentStatus);
		}else{
			wp_send_json_error($aCheckAcceptPaymentStatus);
		}
	}

	public function buyEventPlan($aData=array()){
		// Authentication
		new Billable(array(
			'gateway' => $this->gateway,
			'planID'  => $_POST['eventPlanID']
		));

		$aData = empty($aData) ? $_POST['aData'] : $aData;
		$aData['planID'] = $_POST['eventPlanID'];

		$instReceipt = new Receipt($aData);

		if ( $instReceipt->isFullDiscount ){
			$oPaymentMethod = new NonRecurringPaymentMethod();
			$oCheckout = new Checkout();
			$aPaymentStatus = $oCheckout->begin($instReceipt, $oPaymentMethod);
			$aPaymentStatus['redirectTo'] = get_permalink(PaymentConfiguration::getField('thankyou'));
		}else{
			$oPayPalMethod = new PayPalNonRecurringPaymentMethod();
			$oCheckout = new Checkout();
			$aPaymentStatus = $oCheckout->begin($instReceipt, $oPayPalMethod);
		}

		if ( $aPaymentStatus['status'] == 'success' ){
			wp_send_json_success($aPaymentStatus);
		}else{
			wp_send_json_error($aPaymentStatus);
		}
	}
}