<?php
namespace WilokeListgoFunctionality\Controllers;

use WilokeListgoFunctionality\Framework\Helpers\Time;
use WilokeListgoFunctionality\Framework\Payment\PaymentConfiguration;
use WilokeListgoFunctionality\Model\PaymentMetaModel;
use WilokeListgoFunctionality\Model\PlanRelationshipModel;

class PlanRelationshipController extends Controller {
	public function __construct() {
		add_action('wiloke/wiloke-submission/payment/after_payment', array($this, 'updateSessionID'), 5);
		add_action('wiloke-listgo-functionality/woocommerce/after-order-succeeded', array($this, 'updateSessionIDs'), 1);
		add_action('wiloke_submission/after-orderstatus-changed', array($this, 'updateEmptySessionID'), 1);
	}

	public function updateSessionIDs($aData){
		if ( empty($aData) ){
			return false;
		}

		$this->middleware(['isUserLoggedIn']);

		if ( empty($aData['aOrderMeta']) ){
			return false;
		}

		PlanRelationshipModel::updateEmptySessionID($aData['aOrderMeta']['sessionID'], array(
			'value' => array(
				'planID' => $aData['aOrderMeta']['planID'],
				'userID' => $aData['aOrderMeta']['userID'],
				'ID'     => $aData['aOrderMeta']['planRelationshipID']
			),
			'format' => array(
				'%d',
				'%d',
				'%d'
			)
		));

		if ( !empty($aData['aOrderMeta']['objectID']) ){
			wp_update_post(
				array(
					'ID'             => $aData['aOrderMeta']['objectID'],
					'post_date_gmt'  => Time::getAtomUTCString()
				)
			);
		}
	}

	public function updateEmptySessionID($aData){
		if ( ($aData['newStatus'] != 'succeeded' && $aData['newStatus'] != 'active') || $aData['gateway'] !== 'banktransfer' ){
			return false;
		}

		if ( empty($aData['userID']) || empty($aData['planID']) ){
			return false;
		}

		$planRelationshipID = PaymentMetaModel::get($aData['sessionID'], 'planRelationship');
		if ( empty($planRelationshipID) ){
			return false;
		}

		$this->middleware(['isUserLoggedIn']);
		PlanRelationshipModel::update(
			array(
				'value' => array(
					'sessionID' => $aData['sessionID']
				),
				'format' => array(
					'%d'
				)
			),
			array(
				'value' => array(
					'ID' => $planRelationshipID
				),
				'format' => array(
					'%d'
				)
			)
		);
	}

	/*
	 * Updating sessionID column in Plan Relationship table after the payment has been succeeded
	 *
	 * @param array $aPaymentResponse
	 */
	public function updateSessionID($aPaymentResponse){
		if ( ($aPaymentResponse['status'] != 'succeeded') ){
			return false;
		}

		$this->middleware(['isUserLoggedIn']);

		PlanRelationshipModel::updateEmptySessionID($aPaymentResponse['sessionID'], array(
			'value' => array(
				'planID' => $aPaymentResponse['planID'],
				'userID' => isset($aPaymentResponse['userID']) && !empty($aPaymentResponse['userID']) ? abs($aPaymentResponse['userID']) : get_current_user_id(),
				'ID'     => $aPaymentResponse['planRelationshipID']
			),
			'format' => array(
				'%d',
				'%d',
				'%d'
			)
		));

		if ( !empty($aPaymentResponse['postID']) ){
			if ( $aPaymentResponse['gateway'] != 'banktransfer' ){
				$postStatus = get_post_status($aPaymentResponse['postID']);
				$postType   = get_post_field('post_type', $aPaymentResponse['postID']);

				if ( $postType == 'event' || (PaymentConfiguration::getField('approved_method') != 'manual_review') ){
					$newPostStatus = 'publish';
				}else{
					if ( $postStatus == 'expired' ){
						$newPostStatus = 'renew';
					}else{
						$newPostStatus = 'pending';
					}
				}

				wp_update_post(
					array(
						'ID'            => $aPaymentResponse['postID'],
						'post_status'   => $newPostStatus
					)
				);
			}else{
				wp_update_post(
					array(
						'ID'             => $aPaymentResponse['postID'],
						'post_date_gmt'  => Time::getAtomUTCString()
					)
				);
			}

		}
	}
}