<?php
namespace WilokeListgoFunctionality\Controllers;


use WilokeListgoFunctionality\Framework\Payment\Billable;
use WilokeListgoFunctionality\Framework\Payment\DirectBankTransfer\NonRecurringPaymentMethod;
use WilokeListgoFunctionality\Framework\Payment\PaymentConfiguration;
use WilokeListgoFunctionality\Framework\Payment\Stripe\StripeGenerateToken;
use WilokeListgoFunctionality\Framework\Payment\Stripe\StripeNonRecurringPaymentMethod;
use WilokeListgoFunctionality\Framework\Payment\Checkout;
use WilokeListgoFunctionality\Framework\Payment\Receipt;
use WilokeListgoFunctionality\Framework\Payment\Stripe\StripeRecurringPaymentMethod;
use WilokeListgoFunctionality\Framework\Payment\Stripe\Webhook;
use WilokeListgoFunctionality\Framework\Store\Session;
use WilokeListgoFunctionality\Model\StripeModel;
use WilokeListgoFunctionality\Model\PaymentModel;
use WilokeListgoFunctionality\Framework\Payment\Stripe\StripeCancelPlan;

// new Webhook;
class StripeController extends Controller {
	public $nonRecurringPaymentKey = 'NonRecurringPayment';
	public $recurringPaymentKey = 'RecurringPayment';
	public $gateway = 'stripe';

	public function __construct() {
		add_action('wp_ajax_wiloke_submission_pay_with_stripe', array($this, 'preparePayment'));
		add_action('wiloke_submission/purchase-event-plan-with-stripe', array($this, 'buyEventPlan'));
		add_action('init', array($this, 'listenEvents'));
		add_action('wp_ajax_listgo_cancel_plan_stripe', array($this, 'cancelSubscription'));
	}

	public function cancelSubscription(){
		$this->middleware(['isYourSession'], array(
			'sessionID' => abs($_POST['sessionID'])
		));

		$subscriptionID = StripeModel::getSubscriptionID($_POST['sessionID']);

		if ( empty($subscriptionID) ){
			wp_send_json_error(esc_html__('Sorry, We could not find the subscriptionID of this session. It may the session does not exist or it was cancelled before', 'wiloke'));
		}

		$oCancel = new StripeCancelPlan($subscriptionID);
		$aStatus = $oCancel->cancelPlan();
		
		if ( $aStatus['status'] == 'success' ){
			PaymentModel::updateToCancelledStatusWhereEqualToSessionID($_POST['sessionID']);
			do_action('wiloke-submission/mail/subscription/cancelled', array(
				'customerID' => get_current_user_id(),
				'sessionID'	 => $_POST['sessionID']
			));
			wp_send_json_success(esc_html__('Congratulations! This session has been cancelled.', 'wiloke'));
		}else{
			wp_send_json_error($aStatus['msg']);
		}
	}

	public function listenEvents(){
		if ( !isset($_REQUEST['wiloke-submission-listener']) || ($_REQUEST['wiloke-submission-listener'] != $this->gateway) ){
			return false;
		}

		new Webhook();
	}

	public function preparePayment($aData=array()){
		// Authentication
		new Billable(array(
			'gateway' => $this->gateway,
			'planID'  => Session::getSession(wilokeRepository('sessionkeys:storePlanID'))
		));

		$aData = empty($aData) ? $_POST['aData'] : $aData;
		$aData['planID'] = Session::getSession(wilokeRepository('sessionkeys:storePlanID'));

		$instReceipt = new Receipt($aData);
		if ( PaymentConfiguration::isNonRecurringPayment() ){
			$instPaymentMethod = new StripeNonRecurringPaymentMethod();
		}else{
			$instPaymentMethod = new StripeRecurringPaymentMethod();
		}
		if ( !isset($_POST['redirectTo']) ){
			$instPaymentMethod->thankyouUrl = PaymentConfiguration::thankyouUrl();
		}

		$oCheckout = new Checkout();
		$aPaymentStatus = $oCheckout->begin($instReceipt, $instPaymentMethod);
		
		if ( $aPaymentStatus['status'] == 'success' ){
			$aPaymentStatus['redirectTo'] = $instPaymentMethod->thankyouUrl;
			wp_send_json_success($aPaymentStatus);
		}else{
			wp_send_json_error($aPaymentStatus);
		}
	}

	public function buyEventPlan($aData=array()){
		$aData = empty($aData) ? $_POST['aData'] : $aData;

		// Authentication
		new Billable(array(
			'gateway' => $this->gateway,
			'planID'  => Session::getSession(wilokeRepository('sessionkeys:storePlanID'))
		));

		$aData['planID'] = Session::getSession(wilokeRepository('sessionkeys:storePlanID'));
		$instReceipt = new Receipt($aData);

		if ( $instReceipt->isFullDiscount ){
			$oPaymentMethod = new NonRecurringPaymentMethod();
			$oCheckout = new Checkout();
			$aPaymentStatus = $oCheckout->begin($instReceipt, $oPaymentMethod);
			$aPaymentStatus['redirectTo'] = get_permalink(PaymentConfiguration::getField('thankyou'));
		}else{
			$customerID = StripeModel::getCustomerID(get_current_user_id());

			$oPaymentMethod = new StripeNonRecurringPaymentMethod();
			$oPaymentMethod->thankyouUrl = $aData['redirectTo'];

			if ( empty($customerID) ){
				$aPaymentStatus = $oPaymentMethod->proceedWithSingleToken($instReceipt);
			}else{
				$aPaymentStatus = $oPaymentMethod->proceedPayment($instReceipt);
			}

			$aPaymentStatus['redirectTo'] = $oPaymentMethod->thankyouUrl;
		}

		if ( $aPaymentStatus['status'] == 'success' ){
			wp_send_json_success($aPaymentStatus);
		}else{
			wp_send_json_error($aPaymentStatus);
		}
	}
}