<?php
namespace WilokeListgoFunctionality\Controllers;

use WilokeListgoFunctionality\Framework\Payment\Billable;
use WilokeListgoFunctionality\Framework\Payment\Receipt;
use WilokeListgoFunctionality\Framework\Payment\WooCommerce\WooCommerceNonRecurringPayment;
use WilokeListgoFunctionality\Framework\Store\Session;
use WilokeListgoFunctionality\Model\PaymentModel;
use WilokeListgoFunctionality\Model\PlanRelationshipModel;
use WilokeListgoPromotion\Framework\PromotionConfiguation;

class WooCommerceController extends Controller {
	public $planID;
	public $productID;
	public $orderID;
	public $aSessionIDs;
	public $oReceipt;
	public $gateway = 'woocommerce';

	public function __construct() {
		add_action('wiloke-listgo-functionality/before-redirecting-to-cart', array($this, 'removeProductFromCart'), 10, 1);
		add_filter('woocommerce_add_to_cart_validation', array($this, 'cleanEverythingBeforeAddProductToCart'), 0);
		add_action('woocommerce_checkout_order_processed', array($this, 'newOrderCreated'), 5, 2);
		add_action('woocommerce_order_status_completed', array($this, 'paymentSucceeded'), 5);
		add_action('woocommerce_order_status_pending', array($this, 'paymentPending'), 5);
		add_action('woocommerce_order_status_failed', array($this, 'paymentFailed'), 5);
		add_action('woocommerce_order_status_on-hold', array($this, 'paymentOnHold'), 5);
		add_action('woocommerce_order_status_refunded', array($this, 'paymentRefunded'), 5);
		add_action('woocommerce_order_status_cancelled', array($this, 'paymentCancelled'), 5);
		add_action('woocommerce_order_status_processing', array($this, 'paymentProcessing'), 5);

		add_action('woocommerce_add_to_cart', array($this, 'removeAssociatePlanItems'), 0);
		add_action('wiloke_submission/purchase-event-plan-with-woocommerce', array($this, 'buyEventPlanViaWooCommerce'), 0);
		add_action('woocommerce_thankyou', array($this, 'autoCompleteOrder'), 10, 1 );
	}

	public function autoCompleteOrder($orderID){
		if ( !$orderID ){
			return;
		}

		$order = wc_get_order($orderID);
		$paymentMethod = get_post_meta($orderID, '_payment_method', true);
		// No updated status for orders delivered with Bank wire, Cash on delivery and Cheque payment methods.
		if ( in_array($paymentMethod, array('basc', 'cod', 'cheque')) ){
			return;
		}else {
			$paymentID = PaymentModel::getSessionIDsByWooOrderID($orderID);

			if ( !empty($paymentID) ){
				$order->update_status( 'completed' );
			}
		}
	}

	public function cleanEverythingBeforeAddProductToCart($cart_item_data){
		if ( !isset($_GET['add-to-cart']) || empty($_GET['add-to-cart']) ){
			return $cart_item_data;
		}
		global $woocommerce;

		$planID = PlanRelationshipModel::getPlanIDBuyProductID($_GET['add-to-cart']);
		if ( empty($planID) ){
			return $cart_item_data;
		}

		$woocommerce->cart->empty_cart();
		return true;
	}

	public function removeAssociatePlanItems(){
		global $woocommerce;

		if ($woocommerce->cart->get_cart_contents_count() == 0) {
			return false;
		}

		$productID = Session::getSession(wilokeRepository('sessionkeys:associateProductID'));
		if ( empty($productID) ){
			return false;
		}

		foreach ( $woocommerce->cart->get_cart() as $cartItemKey => $aCardItem ) {
			$planID = PlanRelationshipModel::getPlanIDBuyProductID($productID);
			if ( empty($planID) ){
				continue;
			}

			if($aCardItem['product_id'] != $productID ){
				$woocommerce->cart->remove_cart_item($cartItemKey);
			}
		}
	}

	public function addPlanRelationshipAndOrderIDToCart($aCartItemData, $productID){
		/*Here, We are adding item in WooCommerce session with, wdm_user_custom_data_value name*/

		$planID = PlanRelationshipModel::getPlanIDBuyProductID($productID);
		if ( !empty($planID) ){
			$aNewValues['wiloke_submission']['postID'] = Session::getSession(wilokeRepository('sessionkeys:storePostID'), true);
			$aNewValues['wiloke_submission']['planRelationshipID'] = Session::getSession(wilokeRepository('sessionkeys:storePlanRelationshipIDSessionID'), true);
			if(empty($aCartItemData)){
				return array(
					'meta_data' => $aNewValues
				);
			}

			$aCartItemData = array_merge($aCartItemData['meta_data'], $aNewValues);
			return $aCartItemData;
		}

		return $aCartItemData;
	}

	public function newOrderCreated($orderID, $aData){
		$oOrder = new \WC_Order($orderID);
		$aItems = $oOrder->get_items();

		$category = Session::getSession('category', true);
		$category = $category ? $category : md5('nothing');


		foreach ( $aItems as $aItem ) {
			$productID = $aItem['product_id'];
			if ( $category == 'promotion' ){
				$planID = PromotionConfiguation::getPromotionPricingID();
			}else{
				$planID = PlanRelationshipModel::getPlanIDBuyProductID($productID);
			}

			wc_update_order_item_meta($orderID, 'listgo_category', $category);
			if ( !empty($planID) ){
				$this->preparePayment($planID, $productID, $orderID);
			}
		}
	}

	public function removeProductFromCart($productIDs){
		global $woocommerce;
		foreach ( $woocommerce->cart->get_cart() as $cartItemKey => $aCardItem ) {
			if ( is_array($productIDs) ){
				if( in_array($aCardItem['product_id'], $productIDs) ){
					$woocommerce->cart->remove_cart_item($cartItemKey);
				}
			}else{
				if( $aCardItem['product_id'] == $productIDs ){
					$woocommerce->cart->remove_cart_item($cartItemKey);
				}
			}
		}
	}

	public function updateStatusOfPaymentViaWooCommerce($orderID, $status){
		$aSessionIDs = PaymentModel::getSessionIDsByWooOrderID($orderID);

		if ( empty($aSessionIDs) ){
			return false;
		}

		foreach ($aSessionIDs as $aSession){
			$this->aSessionIDs[] = $aSession['ID'];
			PaymentModel::updatePaymentStatusWhereEqualToSessionID($status, $aSession['ID']);

			if ( $status == 'pending' || $status == 'onhold' ){
				PostsManagementController::migratePostsToExpiredStatus($aSession['ID']);
			}else if ( $status == 'completed' || $status == 'processing' || $status == 'succeeded'  ){
				PostsManagementController::migratePostsToPendingOrPublishStatus($aSession['ID']);
			}else if ( $status == 'refunded' || $status == 'cancelled' ){
				PostsManagementController::migratePostsToDraftStatus($aSession['ID']);
			}
		}
	}

	public function updatePaymentStatusByOrderID($orderID, $status){
		$oOrder = new \WC_Order($orderID);
		$aItems = $oOrder->get_items();

		foreach ( $aItems as $aItem ) {
			$productID = $aItem['product_id'];
			$planID = PlanRelationshipModel::getPlanIDBuyProductID($productID);

			if ( !empty($planID) ){
				$this->updateStatusOfPaymentViaWooCommerce($orderID, $status);
			}
		}
	}

	public function paymentOnHold($orderID){
		$this->updatePaymentStatusByOrderID($orderID, 'onhold');
	}

	public function paymentRefunded($orderID){
		$this->updatePaymentStatusByOrderID($orderID, 'refunded');
	}

	public function paymentCancelled($orderID){
		$this->updatePaymentStatusByOrderID($orderID, 'cancelled');
	}

	public function paymentProcessing($orderID){
		$this->updatePaymentStatusByOrderID($orderID, 'processing');
	}

	public function paymentPending($orderID){
		
		if ( defined('WILOKE_AGREE_WOO_PENDING_STATUS') ){
			$this->paymentSucceeded($orderID);
		}else{
			$this->updatePaymentStatusByOrderID($orderID, 'pending');
		}
	}

	public function paymentFailed($orderID){
		$this->updatePaymentStatusByOrderID($orderID, 'failed');
	}

	public function paymentSucceeded($orderID){
		$oOrder = new \WC_Order($orderID);
		$aItems = $oOrder->get_items();
		$aItemsSuccess = array();
		$aPlanIDs = array();

		$category = wc_get_order_item_meta($orderID, 'listgo_category');
		$category = empty($category) ? md5('nothing') : $category;

		foreach ( $aItems as $aItem ) {
			$productID = $aItem['product_id'];
			$planID = PlanRelationshipModel::getPlanIDBuyProductID($productID);

			if ( !empty($planID) ){
				$aPlanIDs[] = $planID;
				$this->updateStatusOfPaymentViaWooCommerce($orderID, 'succeeded');
				/*
				 * @UserPlanController:setUserPlanOfUserBoughtViaWooCommerce 10
				 */
				do_action('wiloke-listing-functionality/woocommerce/order-succeeded', array(
					'planID'    => $planID,
					'status'    => 'succeeded',
					'gateway'   => $this->gateway,
					'category'  => $category,
					'productID' => $productID,
					'orderID'   => $orderID
				));
			}
		}

		/*
		 * @PlanRelationshipController:updateSessionIDs 5
		 * @UserController:setUserPlanOfUserBoughtViaWooCommerce 20
		 * @PurchasePromotionalPlanController:woocommerceSuccess 25
		 */
		do_action('wiloke-listgo-functionality/woocommerce/after-order-succeeded', array(
			'items'         => $aItemsSuccess,
			'gateway'       => $this->gateway,
			'orderID'       => $orderID,
			'aPlanIDs'      => $aPlanIDs,
			'aSessionIDs'   => $this->aSessionIDs,
			'category'      => $category,
			'aOrderMeta'    => wc_get_order_item_meta($orderID, wilokeRepository('sessionkeys:wooWilokeSubmissionData'), true),
			'billingType'   => wilokeRepository('app:billingTypes', true)->get('nonrecurring')
		));
		$this->updateStatusOfPaymentViaWooCommerce($orderID, 'completed');
	}

	public function updateFailedStatus($aData){
		PaymentModel::updatePaymentStatusWhereEqualToSessionID('failed', $aData['paymentID']);
	}

	public function updateSucceededStatus($aData){
		PaymentModel::updatePaymentStatusWhereEqualToSessionID($aData['status'], $aData['paymentID']);
	}

	public function preparePayment($planID, $productID, $orderID){
		if ( empty($planID) || empty($productID) || empty($orderID) ){
			wp_send_json_error(
				array(
					'msg' => esc_html__('Product ID, Order ID and Plan ID are required', 'wiloke')
				)
			);
		}

		$this->planID     = $planID;
		$this->orderID    = $orderID;
		$this->productID  = $productID;

		$this->oReceipt = new Receipt(array(
			'planID'    => $this->planID,
			'productID' => $this->productID,
			'userID'    => get_current_user_id(),
			'couponCode'=> '',
			'aRequested'=>''
		));
		$this->oReceipt->setupPlan();

		$instWooPayment = new WooCommerceNonRecurringPayment();
		$instWooPayment->setOrderID($this->orderID);
		$instWooPayment->proceedPayment($this->oReceipt);
	}

	public function buyEventPlanViaWooCommerce(){
		// Authentication
		new Billable(array(
			'gateway' => $this->gateway,
			'planID'  => $_POST['eventPlanID']
		));

		$aData = empty($aData) ? $_POST['aData'] : $aData;
		$aData['planID'] = $_POST['eventPlanID'];
		$this->productID = get_post_meta($_POST['eventPlanID'], 'woocommerce_associate_id', true);

		if ( !empty($this->productID) ){
			/*
			 * @hooked WooCommerceController:removeProductFromCart
			 */
			do_action('wiloke-listgo-functionality/before-redirecting-to-cart', $this->productID);

			global $woocommerce;
			$redirectTo = add_query_arg(
				array(
					'add-to-cart' => $this->productID,
					'quantity'    => 1
				),
				$woocommerce->cart->get_cart_url()
			);
			Session::setSession(wilokeRepository('sessionkeys:associateProductID'), $this->productID);
			wp_send_json_success(
				array(
					'msg'           => esc_html__('Congratulations! Your submission has been successfully!', 'wiloke'),
					'redirectTo'    => $redirectTo,
					'isRedirect'    => true
				)
			);
		}else{
			wp_send_json_error(
				array(
					'msg' => esc_html__('The product does not exist', 'wiloke')
				)
			);
		}
	}

}