<?php
namespace WilokeListgoFunctionality\Framework\Helpers;

use Carbon\Carbon;

class Time{
	public static $timezoneString;

	public static function toTimeFormat($time, $format=12){
		return $format == 12 ? date('h:i:A', strtotime($time)) : date('H:i', strtotime($time));
	}

	public static function mysqlDateTime($timestamp=''){
		$timestamp = empty($timestamp) ? current_time('timestamp') : $timestamp;
		return date('Y-m-d H:i:s', $timestamp);
	}

	public static function parseDateFormat($aHour, $format, $key, $extra=''){
		if ( $format == 12 ) {
			$aDate = date( 'h:i:A', strtotime($aHour) );
			$aDate = explode( ':', $aDate );

			$aResult[$extra.$key.'_hour']  = $aDate[0];
			$aResult[$extra.$key.'_minutes'] = $aDate[1];
			$aResult[$extra.$key.'_format'] = $aDate[2];
		}else{
			$aDate = date( 'H:i', strtotime($aHour) );
			$aDate = explode(':', $aDate);
			$aResult[$extra.$key.'_hour']  = $aDate[0];
			$aResult[$extra.$key.'_minutes'] = $aDate[1];
		}

		return $aResult;
	}

	public static function validExtraTime($open,$close){
		if ( $open == $close ){
			return false;
		}

		return true;
	}

	public static function parseBusinessHours($aBusinessHours, $format=12){

		if ( empty($aBusinessHours) || ($aBusinessHours['open_time'] == '00:00:00' && $aBusinessHours['open_time'] == $aBusinessHours['close_time']) ){
			$closedTime = $format == 12 ? '11' : '23';
			return array(
				'extra_open_hour'       => '',
				'extra_open_minutes'    => '',
				'extra_open_format'     => '',
				'extra_close_minutes'   => '',
				'extra_close_hour'      => '',
				'extra_close_format'    => '',
				'closed'                => 0,
				'open_hour'             => apply_filters('wiloke/listgo/default_open_hour', '08'),
				'open_minutes'          => apply_filters('wiloke/listgo/default_star_minutes', '30'),
				'close_hour'            => apply_filters('wiloke/listgo/default_close_hour', $closedTime),
				'close_minutes'         => apply_filters('wiloke/listgo/default_close_minutes', '00'),
				'open_format'           => 'AM',
				'close_format'          => 'PM'
			);
		}

		if ( self::validExtraTime($aBusinessHours['extra_open_time'], $aBusinessHours['extra_close_time']) ){
			$aExtraOpenTime = self::parseDateFormat($aBusinessHours['extra_open_time'], $format, 'open', 'extra_');
			$aExtraClosedTime = self::parseDateFormat($aBusinessHours['extra_close_time'], $format, 'close', 'extra_');
		}else{
			$aExtraOpenTime = array(
				'extra_open_hour'       => '',
				'extra_open_format'     => '',
				'extra_open_minutes'    => ''
			);

			$aExtraClosedTime = array(
				'extra_close_minutes'   => '',
				'extra_close_hour'      => '',
				'extra_close_format'    => ''
			);
		}


		$aOpenTime   = self::parseDateFormat($aBusinessHours['open_time'], $format, 'open');
		$aClosedTime = self::parseDateFormat($aBusinessHours['close_time'], $format, 'close');
		$aClosed['closed'] = $aBusinessHours['closed'];

		return ($aOpenTime + $aClosedTime + $aExtraOpenTime + $aExtraClosedTime + $aClosed);
	}

	/*
	 * Return object $oNow
	 */
	public static function getAtomString(){
		return date(DATE_ATOM, current_time('timestamp'));
	}

	public static function getAtomUTCString(){
		return date(DATE_ATOM, current_time('timestamp', 1));
	}

	public static function timeStampNow(){
		return current_time('timestamp');
	}

	public static function addZeroToTime($time){
		return strlen($time) == 1 ? '0' . $time : $time;
	}

	/*
	 * String To UTC Time
	 *
	 * @param number $timestamp
	 * @return string $date
	 */
	public static function toAtomUTC($timeStamp){
		$timeStamp = is_string($timeStamp) ? strtotime($timeStamp) : $timeStamp;
		date_default_timezone_set("UTC");
		return date(DATE_ATOM, $timeStamp);
	}

	public static function iso8601StartDate(){
		$startDate = date('c', current_time('timestamp'));
		$startDate = date('c', strtotime($startDate  . ' +1 day'));
		$startDate = str_replace('+00:00', 'Z', $startDate);
		return $startDate;
	}

	public static function convertToTwentyFourFormat($hour, $format){
		if ( strtolower($format) == 'pm' ){
			$hour = absint($hour) + 12;
		}

		return $hour;
	}

	/**
	 * Get timestamp UTC now
	 */
	public static function timestampUTCNow(){
		return strtotime(Carbon::now('UTC')->toAtomString());
	}

	public static function convertDayToSeconds($day){
		return $day*24*60*60;
	}

	public static function convertSecondsToDay($seconds, $type='floor'){
		if ( $type == 'floor' ){
			return floor($seconds/(24*60*60));
		}else{
			return ceil($seconds/(24*60*60));
		}
	}
}