<?php
namespace WilokeListgoFunctionality\Framework\Payment\DirectBankTransfer;

use WilokeListgoFunctionality\Framework\Payment\PaymentMethodInterface;
use WilokeListgoFunctionality\Framework\Payment\Coupon;
use WilokeListgoFunctionality\Framework\Store\Session;
use WilokeListgoFunctionality\Model\InvoiceModel;
use WilokeListgoFunctionality\Model\PaymentMetaModel;
use WilokeListgoFunctionality\Model\PaymentModel;

class NonRecurringPaymentMethod implements PaymentMethodInterface{
	use Configuration;
	use GenerateTransactionInfo;

	protected $sessionID;
	protected $aPaymentInfo;
	protected $total;
	protected $discountPrice;

	public function setFocus(){
		$this->isFocus = true;
	}

	public function getBillingType() {
		return wilokeRepository('app:billingTypes', true)->sub('nonrecurring');
	}

	protected function setStatus(){
		if ( $this->oReceipt->isFullDiscount ){
			return 'succeeded';
		}else{
			return 'processing';
		}
	}

	public function proceedPayment($oReceipt){
		$this->setApiContext();
		$this->setReceipt($oReceipt);

		if ( isset($this->oReceipt->aCouponInfo['discountPrice']) && !empty($this->oReceipt->aCouponInfo['discountPrice']) ){
			$this->total = $this->oReceipt->subTotal - $this->oReceipt->aCouponInfo['discountPrice'];
			$this->total = floatval($this->total);
			$this->discountPrice = -$this->oReceipt->aCouponInfo['discountPrice'];
		}else{
			$this->total = $this->subTotal = $this->oReceipt->subTotal;
			$this->discountPrice = 0;
		}

		// Once the session has been completed, We need to record to wiloke_subission_sessions table -> It will generate a sessionID
		$this->insertingNewSession();
		// Payment Meta Data
		$this->insertingNewPaymentMeta();

		/**
		 * @hook PlanRelationship@updateSessionID 5
		 * @hook UserController@createUserPlan 10
		 */
		$planRelationshipID = Session::getSession(wilokeRepository('sessionkeys:storePlanRelationshipIDSessionID'), true);
		do_action('wiloke/wiloke-submission/payment/after_payment', array(
			'planType'          => Session::getSession(wilokeRepository('sessionkeys:planType'), true),
			'gateway'           => $this->gateway,
			'status'            => $this->setStatus(),
			'billingType'       => $this->getBillingType(),
			'sessionID'         => $this->sessionID,
			'planID'            => $oReceipt->planID,
			'planName'          => $oReceipt->planName,
			'postID'            => Session::getSession(wilokeRepository('sessionkeys:storePostID'), true),
			'planRelationshipID'=> $planRelationshipID
		));

		if ( !empty($planRelationshipID) ){
			PaymentMetaModel::set($this->sessionID, 'planRelationship', $planRelationshipID);
		}

		Session::destroySession(wilokeRepository('sessionkeys:storePlanID'));

		if ( !empty($this->discountPrice) ){
			PaymentMetaModel::set($this->sessionID, wilokeRepository('paymentKeys:discountInfo'), array(
				'discountPrice' => $this->discountPrice,
				'couponCode'    => $this->oReceipt->aCouponInfo['discountPrice']
			));
		}

		$invoiceID = 0;
		if ( $this->setStatus() == 'succeeded' ){
			$invoiceID = InvoiceModel::insert(
				array(
					'sessionID'     => $this->sessionID,
					'regular_price' => $this->oReceipt->subTotal,
					'currency'      => $this->aConfiguration['currency_code'],
					'message'       => $this->transactionInfo(),
					'total'         => $this->total,
					'discount'      => $this->discountPrice
				)
			);
		}

		do_action('wiloke-submission/mail', array(
			'type'          => wilokeRepository('mailstatus:type', true)->sub('banktransferProcessing'),
			'gateway'       => $this->gateway,
			'billingType'   => $this->getBillingType(),
			'planID'        => $oReceipt->planID,
			'planName'      => $oReceipt->planName,
			'sessionID'     => $this->sessionID,
			'userID'        => $this->userID,
			'discountPrice' => $this->discountPrice,
			'total'         => $this->total,
			'subTotal'      => $this->oReceipt->subTotal
		));

		return array(
			'status' => 'success',
			'sessionID'=> $this->sessionID,
			'msg'    => esc_html__('Congratulations! Your order has been created successfully', 'wiloke')
		);
	}

	private function insertingNewSession(){
		$this->sessionID = PaymentModel::insert($this, $this->oReceipt, $this->userID, $this->setStatus());
	}

	private function insertingNewPaymentMeta(){
		PaymentMetaModel::set($this->sessionID, wilokeRepository('paymentKeys:info'), $this->transactionInfo());
	}
}