<?php
namespace WilokeListgoFunctionality\Framework\Payment\DirectBankTransfer;


use WilokeListgoFunctionality\Framework\Payment\PaymentMethodInterface;
use WilokeListgoFunctionality\Framework\Store\Session;
use WilokeListgoFunctionality\Model\PaymentMetaModel;
use WilokeListgoFunctionality\Model\PaymentModel;
use WilokeListgoFunctionality\Framework\Payment\PaymentConfiguration;
use WilokeListgoFunctionality\Model\InvoiceModel;

class RecurringPaymentMethod implements PaymentMethodInterface {
	use Configuration;
	use GenerateTransactionInfo;

	protected $sessionID;
	protected $aPaymentInfo;
	protected $relationshipID;
	protected $isTrial = false;
	protected $status = 'processing';

	public function getBillingType() {
		return wilokeRepository('app:billingTypes', true)->sub('recurring');
	}

	public function setFocus(){
		$this->isFocus = true;
	}

	public function proceedPayment($oReceipt){
		$this->setApiContext();
		$this->setReceipt($oReceipt);

		if ( !empty($this->oReceipt->aPlan['trialPeriod']) ){
			$this->isTrial = true;
			$this->status = 'succeeded';
		}

		// Once the session has been completed, We need to record to wiloke_subission_sessions table -> It will generate a sessionID
		$this->insertingNewSession();

		// Payment Meta Data
		$this->insertingNewPaymentMeta();
		$nextBillingDate = '';

		if ( $this->isTrial ){
			InvoiceModel::insert(
				array(
					'sessionID'         => $this->sessionID,
					'regular_price'     => 0,
					'discount'     		=> 0,
					'currency'          => PaymentConfiguration::getCurrency(),
					'message'           => $this->transactionInfo()
				)
			);

			$createdAt = PaymentModel::getCreatedDate($this->sessionID);
			$nextBillingDate = strtotime(date(DATE_ATOM, strtotime($createdAt)) . ' + ' . $oReceipt->aPlan['trialPeriod'] . ' days');
		}
		/**
		 * @hook PlanRelationship@updateSessionID 5
		 * @hook UserController@createUserPlan 10
		 */
		$planRelationshipID = Session::getSession(wilokeRepository('sessionkeys:storePlanRelationshipIDSessionID'), true);
		do_action('wiloke/wiloke-submission/payment/after_payment', array(
			'gateway'           => $this->gateway,
			'status'            => $this->status,
			'billingType'       => $this->getBillingType(),
			'sessionID'         => $this->sessionID,
			'planID'            => $oReceipt->planID,
			'planName'          => $oReceipt->planName,
			'isTrial'       	=> $this->isTrial,
			'nextBillingDate'   => $nextBillingDate,
			'postID'            => Session::getSession(wilokeRepository('sessionkeys:storePostID'), true),
			'planRelationshipID'=> $planRelationshipID
		));

		if ( !empty($planRelationshipID) ){
			PaymentMetaModel::set($this->sessionID, 'planRelationship', $planRelationshipID);
		}

		Session::destroySession(wilokeRepository('sessionkeys:storePlanID'));

		do_action('wiloke-submission/mail', array(
			'type'          => wilokeRepository('mailstatus:type', true)->sub('banktransferProcessing'),
			'gateway'       => $this->gateway,
			'billingType'   => $this->getBillingType(),
			'planID'        => $oReceipt->planID,
			'planName'      => $oReceipt->planName,
			'sessionID'     => $this->sessionID,
			'isTrial'       => $this->isTrial,
			'userID'        => $this->userID
		));

		return array(
			'status'   => 'success',
			'sessionID'=> $this->sessionID,
			'msg'      => esc_html__('Congratulations! Your order has been created successfully', 'wiloke')
		);
	}

	private function insertingNewSession(){
		$this->sessionID = PaymentModel::insert($this, $this->oReceipt, $this->userID, wilokeRepository('app:paymentStatus', true)->sub($this->status));
	}

	private function insertingNewPaymentMeta(){
		PaymentMetaModel::set($this->sessionID, wilokeRepository('paymentKeys:info'), $this->transactionInfo());
	}
}