<?php
namespace WilokeListgoFunctionality\Framework\Payment;


use WilokeListgoFunctionality\Framework\Helpers\GetSettings;

class Receipt{
	public $aInfo = array();
	public $thankyouUrl = null;
	public $cancelUrl   = null;
	public $planID      = null;
	public $planPostType= null;
	public $aPlans    = array();
	public $productID;
	public $planName;
	public $isMultiplePlan = false;
	public $couponID    = '';
	public $total       = 0;
	public $subTotal    = 0;
	public $amountTotal = 0;
	public $planKey = '';
	public $regularPeriod;
	public $isFullDiscount = false;

	public $couponCode  = '';
	public $aCouponInfo = array();

	protected $aPlanSettings = array();

	public function __construct($aInfo, $planKey='') {
		$this->aInfo = $aInfo;
		$this->setupPlan();
//		$this->setPlanID();
		$this->setCoupon();

		$this->setThankyouUrl();
		$this->setCancelUrl();
	}

	public function setupPlan(){
		if ( isset($this->aInfo['isFocusSetUp']) && $this->aInfo['isFocusSetUp'] ){
			return false;
		}

		$this->getPlanID();
		$this->planKey = empty($planKey) ? get_post_field('post_type', $this->planID) : $planKey;
		$this->aPlanSettings = GetSettings::getPostMeta($this->planID, $this->planKey);
		$this->productID = isset($this->aInfo['productID']) ? $this->aInfo['productID'] : '';
		$this->total = $this->getPrice();
		$this->total 	= round($this->total, 2);

		$this->couponCode = isset($this->aInfo['couponCode']) ? $this->aInfo['couponCode'] : '';
		$this->subTotal = $this->total;

		$this->amountTotal = $this->subTotal;
		$this->getPlanName();
		$this->setRegularPeriod();

		if ( empty($this->productID) ){
			$this->aPlan['trialPeriod'] = isset($this->aPlanSettings['trial_period']) ? abs(trim($this->aPlanSettings['trial_period'])) : '';
			
			if ( !empty($this->couponCode) ){
				$instCoupon = new Coupon();
				$instCoupon->getCouponID($this->couponCode);
				$instCoupon->getCouponSlug();
				$instCoupon->getCouponInfo();
				if ( !$instCoupon->isCouponExpired() && $instCoupon->isPostTypeSupported($this->planKey) ){
					$this->couponID = $instCoupon->couponID;
					$this->aCouponInfo = $instCoupon->aSettings;
					$this->aCouponInfo['amount'] = floatval($this->aCouponInfo['amount']);
					$this->aCouponInfo['planName'] = get_the_title($instCoupon->couponID);

					if ( $this->aCouponInfo['type'] == 'percentage' ){
						$this->aCouponInfo['discountPrice'] = round($this->total*$this->aCouponInfo['amount']/100, 2);
					}else{
						$this->aCouponInfo['discountPrice'] = $this->aCouponInfo['amount'];
					}

					if ( $this->aCouponInfo['discountPrice'] == $this->total ){
						$this->isFullDiscount = true;
					}
				}
			}
		}
		

		return $this;
	}

	public function setToken($token){
		$this->aInfo['token'] = $token;
		return $this;
	}

	public function setThankyouUrl(){
		$this->thankyouUrl = isset($this->aInfo['thankyouUrl']) ? $this->aInfo['thankyouUrl'] : '';
		return $this;
	}

	public function setCancelUrl(){
		$this->cancelUrl = isset($this->aInfo['cancelUrl']) ? $this->aInfo['cancelUrl'] : '';
		return $this;
	}

	public function getPlanID(){
		$this->planID = isset($this->aInfo['planID']) ? abs(trim($this->aInfo['planID'])) : '';
		return $this;
	}

	public function getPrice(){
		return round(floatval(trim($this->aPlanSettings['price'])), 2);
	}

	public function setPlanID($planID){
		$this->planID = $planID;
		return $this;
	}

	public function setRegularPeriod(){
		$this->regularPeriod = isset($this->aPlanSettings['regular_period']) ? abs(trim($this->aPlanSettings['regular_period'])) : '';
		return $this;
	}

	public function setTotal($total){
		$this->total = $total;
		return $this;
	}

	public function setSubTotal($subTotal){
		$this->subTotal = $subTotal;
		return $this;
	}

	public function setAmountTotal($amountTotal){
		$this->amountTotal = $amountTotal;
		return $this;
	}

	public function setPlanName($planName){
		$this->planName = $planName;
		return $this;
	}

	public function setMultiplePlans(){
		$this->isMultiplePlan = true;

		foreach ( $this->aInfo['aPlanIDs'] as $planID ){
			$planKey = empty($planKey) ? get_post_field('post_type', $planID) : $planKey;
			$this->aPlanSettings = GetSettings::getPostMeta($planID, $planKey);
			$this->aPlans[$planID]['regular_period'] = isset($this->aPlanSettings['regular_period']) ? abs(trim($this->aPlanSettings['regular_period'])) : '';
			$this->aPlans[$planID]['planName'] = get_the_title($planID);
			$total = $this->getPrice();
			$this->aPlans[$planID]['total'] = $total;
			$this->subTotal += $total;
		}

		$this->subTotal = round($this->subTotal, 2);
		$this->amountTotal = $this->subTotal;
		return $this;
	}

	public function setCoupon(){
		$this->couponID = isset($this->aInfo['couponID']) ? trim($this->aInfo['couponID']) : '';
		return $this;
	}

	public function getPlanName(){
		$this->planName = get_the_title($this->aInfo['planID']);
		return $this;
	}

	public function setCouponCode(){
		$this->couponCode = isset($this->aInfo['couponCode']) ? trim($this->aInfo['couponCode']) : '';
		return $this;
	}
}