<?php
namespace WilokeListgoFunctionality\Framework\Payment\WooCommerce;


use WilokeListgoFunctionality\Framework\Payment\PaymentMethodInterface;
use WilokeListgoFunctionality\Framework\Store\Session;
use WilokeListgoFunctionality\Model\PaymentMetaModel;
use WilokeListgoFunctionality\Model\PaymentModel;

class WooCommerceNonRecurringPayment implements PaymentMethodInterface {
	public $gateway = 'woocommerce';
	protected $userID;
	protected $sessionID;
	protected $oReceipt;
	protected $orderID;
	public $token;

	public function getBillingType() {
		return wilokeRepository('app:billingTypes', true)->sub('nonrecurring');
	}

	protected function setup(){
		$this->token    = time();
		$this->userID   = get_current_user_id();
	}

	private function insertingNewSession(){
		$this->sessionID = PaymentModel::insert($this, $this->oReceipt, $this->userID, 'processing');
	}

	public function setOrderID($orderID){
		$this->orderID = $orderID;
	}

	public function getOrderID(){
		return $this->orderID;
	}

	public function saveWooWilokeSubmissionData($aData=array()){
		$aDefault = array(
			'sessionID'         => $this->sessionID,
			'userID'            => $this->userID,
			'planID'            => $this->oReceipt->planID
		);

		if ( !empty($aData) ){
			$aDefault = array_merge($aDefault, $aData);
		}

		wc_add_order_item_meta($this->orderID, wilokeRepository('sessionkeys:wooWilokeSubmissionData'), $aDefault);
	}

	public function proceedPayment( $oReceipt ) {
		$this->oReceipt = $oReceipt;
		$this->setup();
		$this->insertingNewSession();
		if ( empty($this->sessionID) ){
			wp_send_json_error(
				array(
					'msg' => esc_html__('Could not insert Payment History', 'wiloke')
				)
			);
		}

		PaymentMetaModel::set($this->sessionID, wilokeRepository('paymentKeys:productIDPaymentID'), $this->oReceipt->productID);
		$objectID = Session::getSession(wilokeRepository('sessionkeys:storePostID'), true);
		$planRelationshipID = Session::getSession(wilokeRepository('sessionkeys:storePlanRelationshipIDSessionID'), true);

		$this->saveWooWilokeSubmissionData(array(
			'objectID' => $objectID,
			'planRelationshipID' => $planRelationshipID
		));

		/*
		 * @PaymentStatusController:updatePaymentStatus 5
		 * @PlanRelationshipController:update 5
		 */
		do_action('wiloke/wiloke-submission/payment/after_payment', array(
			'gateway'       => $this->gateway,
			'status'        => 'pending',
			'billingType'   => wilokeRepository('app:billingTypes', true)->sub('nonrecurring'),
			'sessionID'     => $this->sessionID,
			'planID'        => $this->oReceipt->planID,
			'nextBillingDate'=>'',
			'planName'      => get_the_title($this->oReceipt->planID),
			'postID'        => $objectID,
			'planRelationshipID' => $planRelationshipID
		));
	}
}