<?php
namespace WilokeListgoFunctionality\Frontend;

use WilokeListgoFunctionality\AlterTable\AlterTableBusinessHours as BusinessHoursTbl;
use WilokeListgoFunctionality\Framework\Helpers\Time;
class FrontendBusinessHours{
	public $openTime = '';
	public $extraOpenTime = '';
	public $openTimeUTC = '';
	public $extraOpenTimeUTC = '';
	public $alwaysOpen = 'yes';
	public $isClosed = 0;
	public $closeTime = '';
	public $extraClosedTime = '';
	public $closedTimeUTC = '';
	public $extraClosedTimeUTC = '';
	public $dayOfWeek = '';
	public $postID = 0;
	protected $tblName = '';
	protected $isExisted = null;
	protected $metaKey = 'wiloke_listgo_business_hours';
	protected $timezoneString = null;

	public function __construct() {
		add_action('before_delete_post', array($this, 'beforeDeletePosts'), 10);
		add_action('save_post_listing', array($this, 'updateChangeBusinessHours'), 10, 2);
		add_action('wiloke/listgo/front-end/update-business-hour', array($this, 'updateBusinessHours'), 10, 3);
	}

	protected function update(){
		global $wpdb;
		$this->openTimeUTC 			= $this->convertTimeToUTC($this->openTime);
		$this->extraOpenTimeUTC 	= !empty($this->extraOpenTime) ? $this->convertTimeToUTC($this->extraOpenTime) : '';

		$this->closedTimeUTC 		= $this->convertTimeToUTC($this->closeTime);
		$this->extraClosedTimeUTC 	= !empty($this->extraClosedTime) ? $this->convertTimeToUTC($this->extraClosedTime) : '';
		
		if ( empty($this->extraOpenTimeUTC) || empty($this->extraClosedTimeUTC) ){
			$aUpData = array(
				'open_time'     		=> $this->openTime,
				'close_time'    		=> $this->closeTime,
				'open_time_utc' 		=> $this->openTimeUTC,
				'close_time_utc'		=> $this->closedTimeUTC,
				'closed'                => $this->isClosed
			);
			$aPrePare = array('%s', '%s', '%s', '%s', '%s');
		}else{
			$aUpData = array(
				'open_time'     		=> $this->openTime,
				'extra_open_time'   	=> $this->extraOpenTime,
				'close_time'    		=> $this->closeTime,
				'extra_close_time'  	=> $this->extraClosedTime,
				'open_time_utc' 		=> $this->openTimeUTC,
				'extra_open_time_utc' 	=> $this->extraOpenTimeUTC,
				'close_time_utc'		=> $this->closedTimeUTC,
				'extra_close_time_utc'	=> $this->extraClosedTimeUTC,
				'closed'                => $this->isClosed
			);
			$aPrePare = array('%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s');
		}

		return $wpdb->update(
			$this->tblName,
			$aUpData,
			array(
				'post_ID'       => abs($this->postID),
				'day_of_week'   => abs($this->dayOfWeek)
			),
			$aPrePare,
			array(
				'%d',
				'%d'
			)
		);
	}

	protected function insert(){
		global $wpdb;
		$this->openTimeUTC 			= $this->convertTimeToUTC($this->openTime);
		$this->extraOpenTimeUTC 	= $this->convertTimeToUTC($this->extraOpenTime);
		$this->closedTimeUTC 		= $this->convertTimeToUTC($this->closeTime);
		$this->extraClosedTimeUTC 	= $this->convertTimeToUTC($this->extraClosedTime);

		$wpdb->insert(
			$this->tblName,
			array(
				'open_time'     		=> $this->openTime,
				'extra_open_time'   	=> $this->extraOpenTime,
				'close_time'    		=> $this->closeTime,
				'extra_close_time'  	=> $this->extraClosedTime,
				'open_time_utc' 		=> $this->openTimeUTC,
				'extra_open_time_utc' 	=> $this->extraOpenTimeUTC,
				'close_time_utc'		=> $this->closedTimeUTC,
				'extra_close_time_utc'	=> $this->extraClosedTimeUTC,
				'day_of_week'   		=> $this->dayOfWeek,
				'post_ID'       		=> $this->postID,
				'closed'                => $this->isClosed
			),
			array(
				'%s',
				'%s',
				'%s',
				'%s',
				'%s',
				'%s',
				'%s',
				'%s',
				'%d',
				'%d',
				'%s'
			)
		);
	}

	/*
	 * Convert Hours, day of week before inserting/updating
	 * @since 1.0
	 */
	public function prepareData(){
		$this->postID  = abs($this->postID);

		if ( empty($this->postID) ){
			return false;
		}

		$this->openTime  = !empty($this->openTime) ? date('H:i:s', strtotime($this->openTime)) : '';

		if ( !empty($this->extraOpenTime) ){
			$this->extraOpenTime  = date('H:i:s', strtotime($this->extraOpenTime));
		}
		$this->closeTime = !empty($this->closeTime) ? date('H:i:s', strtotime($this->closeTime)) : '';
		if ( !empty($this->extraClosedTime) ){
			$this->extraClosedTime  = date('H:i:s', strtotime($this->extraClosedTime));
		}
		$this->dayOfWeek = abs($this->dayOfWeek);

		$this->isExisted = $this->isValueExist();

		if ( $this->isExisted ){
			$this->update();
		}else{
			$this->insert();
		}
	}

	protected function hasChangeBusinessHours($aBusinessHours, $toggleBusinessHour){
		$this->alwaysOpen = $toggleBusinessHour === 'disable' ? 'no' : 'yes';

		if ( $toggleBusinessHour == 'no' ){
			return false;
		}

		foreach ( $aBusinessHours as $dayOfWeeks => $aData ){
			$this->dayOfWeek = $dayOfWeeks;
			$this->extraOpenTime = $this->extraOpenTimeUTC = $this->extraClosedTime = $this->extraClosedTimeUTC = '';

			$this->isClosed = isset($aData['closed']) ? abs($aData['closed']) : 0;
			$aData['open_minutes'] = strlen($aData['open_minutes']) == 1 ? '0' . $aData['open_minutes'] : $aData['open_minutes'];
			$aData['close_minutes'] = strlen($aData['close_minutes']) == 1 ? '0' . $aData['close_minutes'] : $aData['close_minutes'];

			$aData['open_hour'] = $aData['open_hour'] == '0' ? '00' : $aData['open_hour'];
			$this->openTime  = $aData['open_hour'] . ':' . $aData['open_minutes'];
			$this->openTime = isset($aData['open_format'])  ? $this->openTime . ' ' . $aData['open_format'] : $this->openTime;


			$aData['close_hour'] = $aData['close_hour'] == '0' ? '00' : $aData['close_hour'];
			$this->closeTime = $aData['close_hour'] . ':' . $aData['close_minutes'];
			$this->closeTime = isset($aData['close_format']) ? $this->closeTime . ' ' . $aData['close_format'] : $this->closeTime;

			if ( $this->verifyExtraHours($aData) ){
				$aData['extra_open_minutes'] = strlen($aData['extra_open_minutes']) == 1 ? '0' . $aData['extra_open_minutes'] : $aData['extra_open_minutes'];
				$aData['extra_open_hour'] = $aData['extra_open_hour'] == '0' ? '00' : $aData['extra_open_hour'];
				$this->extraOpenTime = $aData['extra_open_hour'] . ':' . $aData['extra_open_minutes'];

				$this->extraOpenTime = isset($aData['extra_open_format']) ? $this->extraOpenTime . ' ' . $aData['extra_open_format'] : $this->extraOpenTime;

				$aData['extra_close_minutes'] = strlen($aData['extra_close_minutes']) == 1 ? '0' . $aData['extra_close_minutes'] : $aData['extra_close_minutes'];
				$aData['extra_close_hour'] = $aData['extra_close_hour'] == '0' ? '00' : $aData['extra_close_hour'];

				$this->extraClosedTime = $aData['extra_close_hour'] . ':' . $aData['extra_close_minutes'];
				$this->extraClosedTime = isset($aData['extra_close_format']) ? $this->extraClosedTime . ' ' . $aData['extra_close_format'] : $this->extraClosedTime;
			}

			$this->prepareData();
		}
	}

	public function convertTimeToUTC($time){
		if ( $this->timezoneString === null ){
			$aTerms = wp_get_post_terms($this->postID, 'listing_location');
			if ( !empty($aTerms) && !is_wp_error($aTerms) ){
				$oLastTerm = end($aTerms);
				$aLastTermOptions = \Wiloke::getTermOption($oLastTerm->term_id);
				$this->timezoneString = \WilokePublic::timezoneString($aLastTermOptions['timezone']);
			}else{
				$this->timezoneString = get_option('timezone_string');
				$this->timezoneString = \WilokePublic::timezoneString($this->timezoneString);
			}
		}

		date_default_timezone_set($this->timezoneString);
		$time = strtotime($time);
		date_default_timezone_set('UTC');
		$timeUTC   = date('H:i:s', $time);
		return $timeUTC;
	}

	public function updateBusinessHours($aBusinessHours, $toggleBusinessHour, $postID){
		$this->postID = $postID;
		$this->hasChangeBusinessHours($aBusinessHours, $toggleBusinessHour);
	}

	public function verifyExtraHours($aData){
		if ( !isset($aData['extra_open_hour']) || !isset($aData['extra_open_minutes']) || !isset($aData['extra_close_minutes']) || !isset($aData['extra_close_hour']) || $aData['extra_open_hour'] === '' || $aData['extra_open_minutes'] === '' || $aData['extra_close_minutes'] === '' || $aData['extra_close_hour'] === '' ){
			return false;
		}

		return true;
	}

	public function updateChangeBusinessHours($postID, $post){
		if ( $post->post_type !== 'listing' || $post->post_status !== 'publish' ){
			return false;
		}

		$this->postID = $postID;

		if ( !isset($_POST['listgo_bh']) ){
			return false;
		}

		$toggleBusinessHour = sanitize_text_field($_POST['wiloke_toggle_business_hours']);

		$this->hasChangeBusinessHours($_POST['listgo_bh'], $toggleBusinessHour);
	}

	public function beforeDeletePosts($postID){
		if ( get_post_field('post_type', $postID) === 'listing' ){
			$this->postID = absint($postID);
			$this->delete();
		}
	}

	protected function isValueExist(){
		global $wpdb;
		$this->tblName = empty($this->tblName) ? $wpdb->prefix . BusinessHoursTbl::$tblName : $this->tblName;

		$postID = $wpdb->get_var(
			$wpdb->prepare("SELECT {$this->tblName}.post_ID FROM $this->tblName WHERE {$this->tblName}.post_ID=%d AND {$this->tblName}.day_of_week=%d", $this->postID, $this->dayOfWeek)
		);

		if ( empty($postID) ){
			return false;
		}

		return true;
	}

	protected function delete(){
		global $wpdb;
		$this->tblName = empty($this->tblName) ? $wpdb->prefix . BusinessHoursTbl::$tblName : $this->tblName;
		$wpdb->delete(
			$this->tblName,
			array(
				'post_ID'       => $this->postID
			),
			array(
				'%d'
			)
		);
	}
}