<?php
namespace WilokeListgoFunctionality\Frontend;

use WilokeListgoFunctionality\AlterTable\AlterTableEventsCalendar;
use WilokeListgoFunctionality\Framework\Helpers\GetSettings;
use WilokeListgoFunctionality\Framework\Helpers\SetSettings;
use WilokeListgoFunctionality\Framework\Payment\PaymentConfiguration;
use WilokeListgoFunctionality\Frontend\FrontendListingManagement as WilokeListingManage;
use WilokeListgoFunctionality\Model\AddEventPlan;
use WilokeListgoFunctionality\Model\EventModel;
use WilokeListgoFunctionality\Frontend\ConvertTime as WilokeConvertTime;
use WilokeListgoFunctionality\AlterTable\AlterTableGeoPosition;
use WilokeListgoFunctionality\Model\EventsCalendar;
use WilokeListgoFunctionality\Model\StripeModel;

class FrontendEvents{
	public $dateFormat;
	public $eventPosition = 'bottom';

    protected $postType = 'event';
    protected $eventListingRelationshipKey = 'wiloke_listing_event_relationship';
    protected $eventSettingsKey  = 'event_settings';
    protected $eventPublishedAtTimeZoneKey  = 'event_published_at_timezone';
    public $isAllowAddingEvent = null;
    public $aEventStatus = array();
	public static $showEventOnCarouselKey = 'toggle_show_events_on_event_carousel';
	public static $showEventOnListKey = 'toggle_show_events_on_event_listing';
	public static $showEventOnWidgetKey = 'toggle_show_events_on_event_widget';

	public function __construct() {
		add_action('wp_enqueue_scripts', array($this, 'enqueueScripts'), 10);
		add_action('wp_footer', array($this, 'insertAddEventTemplateToFooter'), 10);
		add_action('wiloke/listgo/templates/single-listing/bottom_nav_tab', array($this, 'addEventToButtomOfTab'), 10);
		add_action('wiloke/listgo/templates/single-listing/top_nav_tab', array($this, 'addEventToTopOfTab'), 10);
		add_action('wiloke/listgo/templates/single-listing/before_nav_tab_close', array($this, 'addEventToBottomOfSecondaryTab'), 10);
		add_action('wiloke/listgo/templates/single-listing/before_close_content', array($this, 'printEventContent'), 10);
		add_action('wp_ajax_wiloke_get_event_data', array($this, 'fetchEventData'));
		add_action('wp_ajax_wiloke_fetch_events', array($this, 'fetchEvents'));
		add_action('wp_ajax_nopriv_wiloke_fetch_events', array($this, 'fetchEvents'));
		add_action('wiloke/listgo/single/after_title', array($this, 'renderEventStatusOnSingle'));
		add_action('wiloke/listgo/admin/public/template/vc/listing-layout/after_title', array($this, 'renderEventStatusOnSingle'));
	    add_action('added_post_meta', array($this, 'updateEventCalendar'), 10, 4);
	    add_action('update_post_meta', array($this, 'updateEventCalendar'), 10, 4);
	    add_action('wiloke/listgo/update-event-gmt', array($this, 'updateEventCalendar'), 10, 4);

	    add_action('admin_init', array($this, 'updateMissingEvents'));
	}

	public function updateMissingEvents(){
		global $wpdb;

		if ( get_option('listgo_updated_missing_event') ){
			return false;
		}

		$query = new \WP_Query(
			array(
				'post_type' => 'event',
				'posts_per_page' => -1,
				'post_status'	=> 'publish'
			)
		);

		$eCTbl = $wpdb->prefix.AlterTableEventsCalendar::$tblName;
		$postsTbl = $wpdb->posts;
		if ( $query->have_posts() ){
			global $post;
			while ($query->have_posts()) {
				$query->the_post();

				$status = $wpdb->get_var(
		    	 	$wpdb->prepare(
						"SELECT $eCTbl.eventID FROM $eCTbl LEFT JOIN $postsTbl ON ($eCTbl.eventID=$postsTbl.ID) WHERE $postsTbl.ID=%d",
						$post->ID
					)
		    	);
		    	if ( empty($status) ){
		    		$aEventSettings = get_post_meta($post->ID, 'event_settings', true);
		    		$this->updateEventCalendar(null, $post->ID, 'event_settings', $aEventSettings);
		    	}
			}
			wp_reset_postdata();
		}

		update_option('listgo_updated_missing_event', true);
	}

	public static function generateEventByDateSql($mode){
		global $wpdb;
		$now = current_time('mysql', 1);

		$eCTbl = $wpdb->prefix.AlterTableEventsCalendar::$tblName;
		$postTbl = $wpdb->posts;

		date_default_timezone_set('UTC');
		$starts = date('Y-m-d', time()) . ' 23:59:00';
		$ends = '';

		if ( $mode == 'today' || $mode == 'this_weekend' || $mode == 'tomorrow' ){
			if ( $mode == 'tomorrow' ){
				$starts = date('Y-m-d H:i:s', strtotime($starts . ' + 1 days'));
			}elseif ( $mode == 'this_weekend' ){
				$starts = date('Y-m-d H:i:s', strtotime('sunday this week'));
			}else{
				$starts = $now;
			}

			$sql = $wpdb->prepare(
				"SELECT SQL_CALC_FOUND_ROWS $eCTbl.eventID FROM $eCTbl LEFT JOIN $postTbl ON ($eCTbl.eventID=$postTbl.ID) WHERE $postTbl.post_status=%s AND %s >= $eCTbl.startsGMT AND $eCTbl.endsOnGMT >= %s",
				'publish', $starts, $now
			);
			// wp_send_json_success($sql);
		}else{
			if ( $mode == 'next_week' ){
				$starts = date('Y-m-d H:i:s', strtotime('monday next week'));
				$ends = date('Y-m-d H:i:s', strtotime('sunday next week'));
			}else if($mode == 'this_week'){
				$starts = date('Y-m-d H:i:s', strtotime('monday this week'));
				$ends = date('Y-m-d H:i:s', strtotime('sunday this week'));
			}else if($mode == 'next_month'){
				$starts = date('Y-m-d H:i:s', strtotime('first day of next month'));
				$ends = date('Y-m-d H:i:s', strtotime('last day of next month'));
			}else{
				$starts = date('Y-m-d H:i:s', strtotime('first day of next week'));
				$ends = date('Y-m-d H:i:s', strtotime('last day of next week'));
			}

			$sql = $wpdb->prepare(
				"SELECT SQL_CALC_FOUND_ROWS $eCTbl.eventID FROM $eCTbl LEFT JOIN $postTbl ON ($eCTbl.eventID=$postTbl.ID) WHERE $postTbl.post_status=%s AND $eCTbl.endsOnGMT <= %s AND $eCTbl.endsOnGMT >= %s AND $eCTbl.endsOnGMT >= %s",
				'publish', $ends, $now, $starts
			);
		}

		return array(
			'sql'   => $sql,
			'starts'=> esc_sql($starts),
			'ends'  => esc_sql($ends),
			'mode'  => esc_sql($mode)
		);
    }

	public function updateEventCalendar($metaID, $eventID, $metaKey, $aEventSettings){
		if ( $metaKey !== 'event_settings' ){
			return false;
		}

		$postType = get_post_field('post_type', $eventID);
		if ( $postType != 'event' ){
			return false;
		}

		$postStatus = get_post_field('post_status', $eventID);
	    if ( $postStatus !== 'publish' ){
	        return false;
        }
        $starts = $ends = '';

		date_default_timezone_set('UTC');
		if ( isset($aEventSettings['start_on']) ){
			if ( !isset($aEventSettings['start_at']) || empty($aEventSettings['start_at']) ){
				$starts = strtotime(trim($aEventSettings['start_on']));
			}else{
				$starts = strtotime(trim($aEventSettings['start_on']) . ' ' . $aEventSettings['start_at']);
			}
		}
		
		if ( isset($aEventSettings['end_on']) ){
			if ( !isset($aEventSettings['end_at']) || empty($aEventSettings['end_at']) ){
				$ends = strtotime(trim($aEventSettings['end_on']));
			}else{
				$ends = strtotime(trim($aEventSettings['end_on']) . ' ' . $aEventSettings['end_at']);
			}
		}
		

		if ( empty($starts) || empty($ends) ){
			return false;
		}

		$starts = date('Y-m-d H:i:s', $starts);
		$ends = date('Y-m-d H:i:s', $ends);
        EventsCalendar::insert($eventID, $starts, $ends);
    }

	public static function isEnableAddEvent(){
	    $toggleAddEvent = PaymentConfiguration::getField('toggle_add_event');
	    if ( isset($toggleAddEvent) && $toggleAddEvent == 'disable' ){
	        return false;
        }

        return true;
    }

	public function getLatestEventStatus($post){
		global $wpdb;

		$postsTbl 	= $wpdb->posts;
		$eventsTbl 	= $wpdb->prefix . AlterTableEventsCalendar::$tblName;

		$latestEventID = $wpdb->get_var(
			$wpdb->prepare(
				"SELECT $postsTbl.ID FROM ".$postsTbl." LEFT JOIN $eventsTbl ON ($postsTbl.ID=$eventsTbl.eventID) WHERE $postsTbl.post_parent=%d AND $postsTbl.post_status=%s AND $postsTbl.post_type='event' AND $eventsTbl.endsOnGMT >= UTC_TIMESTAMP ORDER BY $eventsTbl.startsGMT ASC",
				$post->ID, 'publish'
			)
		);

		if ( empty($latestEventID) ){
		    if ( !self::isEnableAddEvent() ){
			    return array(
				    'status'     => 'disable'
			    );
            }else{
			    return array(
				    'status'     => 'create',
				    'name'       => __('Create', 'wiloke'),
				    'longname'   => __('Create Event', 'wiloke'),
			    );
            }
		}

		$oEvent = get_post($latestEventID);

		$aEventStatus = EventModel::checkEventStatus($oEvent);
		return $aEventStatus;
	}

	public function renderEventStatusOnSingle($post){
		$aLatestEventStatus = $this->getLatestEventStatus($post);
        ?>
        <?php if ( ($aLatestEventStatus['status'] == 'upcomming') || ($aLatestEventStatus['status'] == 'ongoing') ) : ?>
        <span class="listing__icon-notif <?php echo esc_attr($aLatestEventStatus['status']); ?>" data-tooltip="<?php echo esc_html($aLatestEventStatus['longname']); ?>"><i class="fa fa-bullhorn"></i></span>
        <?php endif; ?>
        <?php
    }

	public function maybeAllEvenInTrash($aEvents){
	    if ( empty($aEvents) ){
	        return true;
        }

	    foreach ( $aEvents as $eventID ){
	        if ( !empty($eventID) && get_post_field('post_status', $eventID) == 'publish' ){
	            return false;
            }
        }

        return true;
    }

	public function fetchEvents(){
	    if ( empty($_POST['paged']) || empty($_POST['configuration']) ){
	        wp_send_json_error(array(
	                'msg' => WilokeListingManage::message(
                        array(
                            'message' => esc_html__('Whoops! Sorry, We can\'t find what you are looking for.', 'wiloke')
                        ),
                        'danger',
                        true
                    )
                )
            );
        }
		$aConfiguration = json_decode(stripslashes($_POST['configuration']), true);
		$aConfiguration['posts_per_page'] = abs($aConfiguration['posts_per_page']);
		$aConfiguration['posts_per_page'] = !empty($aConfiguration['posts_per_page']) && $aConfiguration['posts_per_page'] <= 30  ? $aConfiguration['posts_per_page'] : get_option('posts_per_page');

        $aArgs = array(
	        'post_type'     => 'event',
	        'post_status'   => 'publish',
	        'orderby'		=> 'menu_order post_date',
            'paged'         => abs($_POST['paged'])
        );

        if ( !empty($_POST['sDate']) ){
            $aArgs['orderby'] = 'event_'.$_POST['sDate'];
        }

        $atts['limit_character'] = !isset($_POST['limitCharacter']) || empty($_POST['limitCharacter']) ? 100 : $_POST['limitCharacter'];

        $errMsg = WilokeListingManage::message(
	        array(
		        'message' => esc_html__('Whoops! Sorry, We can\'t find what you are looking for.', 'wiloke')
	        ),
	        'danger',
            true
        );

		if ( !empty($_POST['catID']) ){
            $aArgs['tax_query'] = array(
                array(
                    'taxonomy' => 'listing_cat',
                    'field'    => 'term_id',
                    'terms'    => absint($_POST['catID'])
                )
            );
        }

		if ( !empty($_POST['latLng']) ){
		    $aThemeOptions = \Wiloke::getOption('wiloke_themeoptions');
			$distance = isset($_POST['distance']) && !empty($_POST['distance']) ? abs($_POST['distance']) : 20;
			$aArgs['geocode'] = array(
				'unit' => isset($aThemeOptions['listgo_search_default_unit']) ? $aThemeOptions['listgo_search_default_unit'] : 'km',
                'latLng' => $_POST['latLng'],
                'radius' => $distance
			);
		}

        if ( empty($_POST['catID']) && !empty($_POST['s']) ){
	        $aArgs['s'] = trim($_POST['s']);
        }
        $aArgs['posts_per_page'] = $aConfiguration['posts_per_page'];
        
        $query = new \WP_Query($aArgs);

        if ( $query->have_posts() ){
            global $post;
            ob_start();
            while ($query->have_posts()){
                $query->the_post();
                include WILOKE_PUBLIC_DIR . 'template/vc/events-layout/default.php';
            }
            $content = ob_get_contents();
            ob_end_clean();
        }else{
            wp_send_json_error(
                array(
                    'msg' => $errMsg
                )
            );
        }

        wp_send_json_success(
            array(
                'msg'       => $content,
                'totalPosts'=>  $query->found_posts,
                'maxPages'  => $query->max_num_pages
            )
        );
    }

	public static function searchForm(){
		$aCategories = \Wiloke::getTaxonomyHierarchy('listing_cat');
		?>

        <form method="GET" id="listgo-event-searchfrom" class="form form--listing">
            <div class="col-sm-4">
                <div class="form-item item--search">
                    <label for="s_search" class="label"><?php echo esc_html_e('Enter event name', 'wiloke'); ?></label>
                    <span class="input-text input-icon-inside">
                        <input id="s_search" class="disabled-autocomplete-ajax" type="text" name="listgo_event_name_or_cat" value="">
                        <?php
                        if ( !empty($aCategories) && !is_wp_error($aCategories) ) :
                            ?>
                            <input type="hidden" id="wiloke-original-search-suggestion" value="<?php echo esc_attr(json_encode($aCategories)); ?>">
                            <input type="hidden" id="s_listing_cat" name="s_listing_cat" value="">
                        <?php endif; ?>
                        <i class="input-icon icon_search"></i>
                    </span>
                </div>
            </div>
            <div class="col-sm-4">
                <?php
                    \WilokeSearchSystem::renderLocationField(esc_html__('Location', 'wiloke'), '');
                ?>
            </div>

            <?php
            $aDates = array(
                ''              => esc_html__('All Date', 'wiloke'),
                'today'         => esc_html__('Today', 'wiloke'),
                'tomorrow'      => esc_html__('Tomorrow', 'wiloke'),
                'this_week'     => esc_html__('This Week', 'wiloke'),
                'this_weekend'  => esc_html__('This Weekend', 'wiloke'),
                'next_week'     => esc_html__('Next Week', 'wiloke'),
                'next_month'    => esc_html__('Next Month', 'wiloke'),
            );
            $sDate = isset($_REQUEST['s_date']) ? $_REQUEST['s_date'] : '';
            ?>
            <div class="col-sm-4">
                <div class="form-item item--search">
                    <label for="s_date" class="label"><?php echo esc_html_e('Date', 'wiloke'); ?></label>
                    <span class="input-text input-icon-inside">
                        <select name="s_date" id="s_date">
                            <?php foreach ($aDates as $key => $date) : ?>
                            <option <?php selected($sDate, $key); ?> value="<?php echo esc_attr($key); ?>"><?php echo esc_html($date); ?></option>
                            <?php endforeach; ?>
                        </select>
                    </span>
                </div>
            </div>


        </form>
        <?php
    }

    public static function paymentMethods(){
	    $aPaymentGateWays = PaymentConfiguration::getGatewaySettings();
	    if ( !empty($aPaymentGateWays) ):
	        ?>
            <table class="addlisting-popup__package">
			    <?php
                $order = 0; $isIncludedForm = false;
			    foreach ($aPaymentGateWays as $gateway => $aInfo) :
                    $checked = empty($order) ? 'checked' : '';
				    ?>
                    <tr id="listgo-gateway-<?php echo esc_attr($gateway); ?>" class="listgo-gateway-settings">
                        <td class="addlisting-popup__package-action" data-title="<?php echo esc_attr($aInfo['text']); ?>">
                            <label class="addlisting-popup__radio">
                                <input type="radio" name="event_payment_method" value="<?php echo esc_attr($gateway); ?>" <?php echo esc_attr($checked); ?>><span></span>
                            </label>
                        </td>
                        <td class="addlisting-popup__package-description">
                            <div class="addlisting-popup__package-name">
                                <strong><?php echo esc_html($aInfo['text']); ?></strong>
                            </div>
                        </td>
                        <?php if ( ($gateway == '2checkout' || ($gateway == 'stripe' && empty(StripeModel::getCustomerID(get_current_user_id()))) ) ) : ?>
                        <td class="addlisting-popup__package-form">
                            <div id="wiloke-card-for-<?php echo esc_attr($gateway); ?>" class="wiloke-event-method__form">
	                            <?php
                                if ( !$isIncludedForm ){
	                                $isIncludedForm = true;
	                                include plugin_dir_path(__FILE__) . 'templates/card-form.php';
                                }
                                ?>
                            </div>
                        </td>
                        <?php endif; ?>
                    </tr>
				    <?php
                $order++;
			    endforeach;
			    ?>
                <tr id="listgo-event-coupon" class="listgo-gateway-settings">
                    <td class="addlisting-popup__package-action">
                        <?php esc_html_e('Apply Coupon', 'wiloke'); ?>
                    </td>
                    <td class="addlisting-popup__package-description">
                        <input id="listgo-apply-coupon-code" type="text" name="couponCode" placeholder="<?php esc_html_e('Enter your coupon code', 'wiloke'); ?>" value="">
                        <input id="listgo-has-coupon" type="hidden" name="hasCoupon" value="no">
                        <i id="listgo-coupon-message"></i>
                    </td>
                </tr>
            </table>
            <?php
        else:
            WilokeListingManage::message(
                array(
                    'msg' => esc_html__('Oops! The website has not any payment gateway yet.', 'wiloke')
                )
            );
        endif;
    }

	public static function fetchEventPlan(){
        $query = new \WP_Query(
            array(
                'post_type'      => 'event-pricing',
                'posts_per_page' => -1,
                'post_status'    => 'publish',
                'orderby'        => 'menu_order'
            )
        );

        if ( $query->have_posts() ):
            $order = 0;
            ?>
            <table class="addlisting-popup__package">
            <?php
            while ( $query->have_posts() ) :
                $query->the_post();
                if ( empty($order) ){
	                $activeClass = 'active';
	                $checked = 'checked';
                }else{
	                $activeClass = '';
	                $checked = '';
                }
                $aEventSettings = \Wiloke::getPostMetaCaching($query->post->ID, 'event_pricing_settings');

                if ( empty($aEventSettings['price']) || empty($aEventSettings['number_of_posts']) ){
		            return '';
	            }

	            $productID = get_post_meta($query->post->ID, 'woocommerce_associate_id', true);
                ?>
                    <tr class="<?php echo esc_attr($activeClass); ?>" data-title="<?php echo esc_attr($query->post->ID); ?>">
                        <td class="addlisting-popup__package-action" data-title="<?php echo esc_attr($query->post->post_title); ?>">
                            <label class="addlisting-popup__radio">
                                <input type="radio" name="event_plan_id" data-productid="<?php echo esc_attr($productID); ?>" value="<?php echo esc_attr($query->post->ID); ?>" <?php echo esc_attr($checked); ?>><span></span>
                            </label>
                        </td>

                        <td class="addlisting-popup__package-price-td" data-title="<?php esc_html_e('Price', 'wiloke'); ?>">
                            <span class="addlisting-popup__package-price">
                                <sup class="addlisting-popup__package-price-currency color-yelow"><?php echo esc_html(PaymentConfiguration::getCurrency()); ?></sup>
                                <span class="addlisting-popup__package-price-amount color-yelow"><?php echo esc_html($aEventSettings['price']); ?></span>
                                <span class="addlisting-popup__package-price-slash">/</span>
                                <span class="addlisting-popup__package-price-time color-green"><?php echo esc_html($aEventSettings['number_of_posts']); ?> <?php echo absint($aEventSettings['number_of_posts']) > 1 ? esc_html__('Events', 'wiloke') : esc_html__('Event', 'wiloke'); ?></span>
                            </span>
                        </td>
                        <td class="addlisting-popup__package-description" data-title="<?php esc_html_e('Description', 'wiloke'); ?>">
                            <div>
                                <strong><?php echo esc_html($query->post->post_title); ?></strong>
                                <p><?php echo get_the_content($query->post->ID); ?></p>
                            </div>
                        </td>
                    </tr>
                <?php
	            $order++;
            endwhile;
            ?>
            </table>
            <?php
        endif;
        wp_reset_postdata();
    }

    public function fetchEventData(){
	    if ( empty($_POST['security']) || empty($_POST['eventID']) || !check_ajax_referer('wiloke-nonce', 'security', false) ){
		    wp_send_json_error(
			    array(
				    'msg' => esc_html__('Wrong Security Code', 'wiloke')
			    )
		    );
	    }

	    if ( $msg = $this->eventExpirationEdit($_POST['eventID']) ){
		    wp_send_json_error(
			    array(
				    'msg' => $msg
			    )
		    );
	    }

        if ( !current_user_can('edit_theme_options') && (get_post_field('post_author', $_POST['eventID']) != get_current_user_id()) ){
	        wp_send_json_error(
		        array(
			        'msg' => esc_html__('You don\'t have permission to access this area', 'wiloke')
		        )
	        );
        }

        $aData['event_title'] = get_the_title($_POST['eventID']);
        $aData['event_content'] = get_post_field('post_content', $_POST['eventID']);
        $aData['event_featured_image'] = get_post_thumbnail_id($_POST['eventID']);
        $aData['event_featured_image_url'] = !empty($aData['event_featured_image']) ? get_the_post_thumbnail_url($_POST['eventID'], 'large') : '';

        $aEventSettings = \Wiloke::getPostMetaCaching($_POST['eventID'], $this->eventSettingsKey);

        $aData = array_merge($aData, $aEventSettings);

        wp_send_json_success($aData);
    }

    public function eventExpirationEdit($eventID, $compareWith=null){
        if ( current_user_can('edit_theme_options') ){
            return false;
        }

	    $aEventSettings = \Wiloke::getPostMetaCaching($eventID, $this->eventSettingsKey);
	    $timeZone = '';
	    if ( !empty($aEventSettings['belongs_to']) ){
		    $aLocation = wp_get_post_terms($aEventSettings['belongs_to'], 'listing_location');
		    if ( !empty($aLocation) && !is_wp_error($aLocation) ){
			    $oLocation = end($aLocation);
			    $aLocationData = \Wiloke::getTermOption($oLocation->term_id);
			    if ( isset($aLocationData['timezone']) ){
				    $timeZone = $aLocationData['timezone'];
			    }
		    }
	    }

	    $createdAt = get_post_meta($eventID, $this->eventPublishedAtTimeZoneKey, true);
	    if ( empty($createdAt) ){
			$createdAt = get_post_field('post_date_gmt', $eventID);
	    }
	    
	    $createdAt = strtotime($createdAt);
	    date_default_timezone_set(\WilokePublic::timezoneString($timeZone));
	    $now = time();
	    $compareWith = empty($compareWith) ? 2100 : $compareWith;
	    if ( intval($now - $createdAt) >= absint($compareWith)  ){
		    return true;
	    }

	    return false;
    }

    public function allowAddingEvent($post){
        if ( $this->isAllowAddingEvent !== null ){
            return $this->isAllowAddingEvent;
        }

        if ( current_user_can('edit_theme_options') ){
	        $this->isAllowAddingEvent = true;
	        return true;
        }

        if ( get_current_user_id() != $post->post_author ){
            $this->isAllowAddingEvent = false;
            return false;
        }

	    $this->isAllowAddingEvent = $post->post_status === 'publish';
        return $this->isAllowAddingEvent;
    }
    
	protected function parseEventData($aRawData){
		$aEventSettings = json_decode(urldecode(base64_decode($aRawData['data'])), true);
		$aParsedData = array();
		foreach ( $aEventSettings as $aField ){
			$name = strip_tags($aField['name']);
			$aParsedData[$name] = sanitize_text_field($aField['value']);
		}
		$aParsedData['event_content'] = wp_kses_post($aRawData['event_content']);

		return $aParsedData;
	}

	public function enqueueScripts(){
	    global $post;
		if ( !is_singular('listing') || ( !current_user_can('edit_theme_options') && ($post->post_author != get_current_user_id()) ) ){
			return false;
		}

		if ( PaymentConfiguration::checkGateway('2checkout') ){
		    wp_enqueue_script('2checkout', 'https://www.2checkout.com/checkout/api/2co.min.js', array('jquery'), WILOKE_LISTGO_FC_VERSION, true );
        }
		

		wp_enqueue_script('jquery-ui-datepicker');
		wp_enqueue_style('jquery-ui', '//code.jquery.com/ui/1.12.1/themes/base/jquery-ui.min.css');
		wp_enqueue_style('wiloke-shortcode-popup', get_template_directory_uri() . '/css/popup-css.css', array(), WILOKE_LISTGO_FC_VERSION);
		wp_enqueue_script('listgo-add-events', WILOKE_LISTGO_FUNC_URL . 'public/source/js/addevent.js', array('jquery'), WILOKE_LISTGO_FC_VERSION, true);
		wp_enqueue_script('wiloke-listgo-coupon', WILOKE_LISTGO_FUNC_URL . 'public/source/js/coupon.js', array('jquery'), WILOKE_LISTGO_FC_VERSION, true);

		wp_localize_script('listgo-add-events', 'WILOKE_LISTGO_EVENTS', array(
            'date_format' => get_option('date_format'),
            'saved_btn'   => esc_html__('Saved', 'wiloke'),
            'select_btn'  => esc_html__('Select', 'wiloke'),
            'error_btn'   => esc_html__('Error', 'wiloke'),
            'create_btn'  => esc_html__('Create Event', 'wiloke'),
            'update_event'=> esc_html__('Update Event', 'wiloke'),
            'pay_btn'     => esc_html__('Pay & Publish', 'wiloke'),
            'twoCheckoutMissingCardData' => WilokeListingManage::message(
	            array(
		            'message' => esc_html__('Please expand the card form and complete all your information', 'wiloke'),
		            'icon' => 'icon_error-triangle_alt'
	            ),
	            'danger',
	            true
            )
        ));
	}

	public function addEventToSecondaryTab($post){
	    if ( !empty($this->aEventStatus) ) :
        ?>
        <li class="tab-nav-event <?php echo esc_attr($this->aEventStatus['status']); ?>"><a href="#tab-event"><?php echo esc_html($this->aEventStatus['longname']); ?></a></li>
        <?php
        endif;
    }

    public function addEventToBottomOfSecondaryTab(){
    	if ( $this->getEventPosition() != 'bottom' ){
    		return '';
    	}
    	$this->addEventToSecondaryTab($post);
    }

    public function addEventToTopOfSecondaryTab(){
    	if ( $this->getEventPosition() != 'top' ){
    		return '';
    	}
    	$this->addEventToSecondaryTab($post);
    }

    public function getEventPosition(){
    	return apply_filters('listgo/wiloke-listgo-functionality/single-listing/event-tab-position', $this->eventPosition);
    }

    public function addEventToTopOfTab($post){
    	if ( $this->getEventPosition() != 'top' ){
    		return '';
    	}
    	$this->addEventTab($post, 'top');
    }

    public function addEventToButtomOfTab($post){
    	if ( $this->getEventPosition() != 'bottom' ){
    		return '';
    	}
    	$this->addEventTab($post);
    }

	public function addEventTab($post, $position='bottom'){
		if ( $post->post_status !== 'publish' ){
		    return '';
        }

        $aEventStatus = $this->getLatestEventStatus($post);
        
		if (  $aEventStatus['status'] == 'disable' ){
		    return '';
        }

        if ( $position == 'top' ){
        	$additionalClass = 'active';
        }else{
        	$additionalClass = '';
        }

		$this->aEventStatus = $aEventStatus;
		if ( !$this->allowAddingEvent($post) ):
            if ( $aEventStatus['status'] !== 'create' ) :
	            $this->aEventStatus = '';
        ?>
			<li class="tab-nav-event <?php echo esc_attr($aEventStatus['status']) . ' ' . $additionalClass; ?>"><a href="#tab-event"><?php echo esc_html($aEventStatus['longname']); ?></a></li>
		<?php
            endif;
		else:
            if ( $aEventStatus['status'] == 'create' ) :
		?>
			<li class="<?php echo esc_attr($additionalClass); ?> tab-nav-event zero-event"><a href="#tab-event"><span id="listgo-add-event" class="add-event">+</span> <?php esc_html_e('Create Event', 'wiloke'); ?> <span class="wil-sos"></span></a></li>
		<?php
            else:
            ?>
                <li class="<?php echo esc_attr($additionalClass); ?> tab-nav-event <?php echo esc_attr($aEventStatus['status']); ?>"><a href="#tab-event"><?php echo esc_html($aEventStatus['longname']); ?> <span class="wil-sos"></span></a></li>
            <?php
            endif;
		endif;
	}

	public function printEventContent($post){
		if ( $post->post_status !== 'publish' ){
			return '';
		}

		global $wpdb;

		$eventsCalendarTbl = $wpdb->prefix . AlterTableEventsCalendar::$tblName;
		$postsTbl = $wpdb->posts;

		$aRawEventsIDs = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT DISTINCT $postsTbl.ID FROM $postsTbl LEFT JOIN $eventsCalendarTbl ON ( $postsTbl.ID = $eventsCalendarTbl.eventID ) WHERE $postsTbl.post_type='event' AND $postsTbl.post_status='publish' AND $postsTbl.post_parent=%d AND $eventsCalendarTbl.endsOnGMT >= UTC_TIMESTAMP ORDER BY $eventsCalendarTbl.startsGMT ASC",
				$post->ID
			),
			ARRAY_A
		);
		
		if ( empty($aRawEventsIDs) || is_wp_error($aRawEventsIDs) ){
		    return '';
        }

        $aEventIDs = array_map(function($aData){
        	return $aData['ID'];
        }, $aRawEventsIDs);

        $query = new \WP_Query(
	        array(
                'post_type'        	=> $this->postType,
                'post__in'			=> $aEventIDs,
                'posts_per_page'   	=> 100,
                'orderby'          	=> 'post__in'
            )
        );

        $status = $this->getEventPosition() == 'top' ? 'active' : '';
		if ( !current_user_can('edit_theme_options') && ($post->post_author != get_current_user_id()) ): ?>
            <div id="tab-event" class="tab__panel <?php echo esc_attr($status); ?>">
                <?php
                $order = 0;
                global $post;
                while ($query->have_posts()){
                    $query->the_post();
	                $isPrintAddListingBtn = ($order == 0);
	                $this->renderEvent($isPrintAddListingBtn);
	                $order++;
                }
                wp_reset_postdata();
                ?>
            </div>
        <?php else: ?>
            <div id="tab-event" class="tab__panel <?php echo esc_attr($status); ?>">
                <?php
                $order = 0;
                global $post;
                while ($query->have_posts()){
	                $query->the_post();
	                $isPrintAddListingBtn = ($order == 0);
	                $this->renderEvent($isPrintAddListingBtn);
	                $order++;
                }
                wp_reset_postdata();
                ?>
            </div>
        <?php endif;
    }

    public function renderDate($date){
    	$dateToTimeStamp = strtotime($date);
    	if ( empty($this->dateFormat) ){
    		$this->dateFormat = get_option('date_format');
    	}

    	return date_i18n($this->dateFormat, $dateToTimeStamp);
    }

    public function renderEvent($isPrintAddListingBtn=true){
	    global $post, $wiloke;
	    $timeFormat = !isset($wiloke->aThemeOptions['time_format']) || $wiloke->aThemeOptions['time_format'] == 24 ? 'H:i' : 'h:i A';
	    $aEventSettings = \Wiloke::getPostMetaCaching($post->ID, $this->eventSettingsKey);
	    if ( !$this->maybeAllEvenInTrash($aEventSettings) && $isPrintAddListingBtn ) :
		    if ( self::isEnableAddEvent() ):
            $remainingEvent = AddEventPlan::getUserRemainingEvents(get_post_field('post_author', $post->ID));
	            if ( current_user_can('edit_theme_options') || (get_post_field('post_author', $post->ID) == get_current_user_id()) ) :
            ?>
                    <a id="listgo-add-event" class="listing-single__event-create add-event inline-add-event" href="#" data-remaining="<?php echo esc_attr($remainingEvent); ?>"><i class="icon_plus"></i> <?php esc_html_e('Add Event', 'wiloke'); ?></a>
            <?php endif;
            endif;
        endif; ?>

        <div id="event-<?php echo esc_attr($post->ID); ?>" class="listing-single__event">
		    <?php if ( has_post_thumbnail($post->ID) ) : ?>
                <div class="listing-single__event-media">
				    <?php echo get_the_post_thumbnail($post->ID, 'large'); ?>
                </div>
		    <?php endif; ?>

            <h2 class="listing-single__event-title"><?php the_title(); ?></h2>

            <div class="listing-event__start">
                <table class="listing-event__table">
                    <thead>
                        <tr>
                            <th><?php echo esc_html__(apply_filters('wiloke-listgo-functionality/public/event-list/address-field-name', esc_html__('Address', 'wiloke'))); ?></th>
                            <th><?php esc_html_e('From', 'wiloke'); ?></th>
                            <th><?php esc_html_e('To', 'wiloke'); ?></th>
                        </tr>
                    </thead>
                    <tr>
                        <td class="listing-event__address" title="<?php esc_html_e('Address', 'wiloke'); ?>">
                            <p>
                                <i class="color-yelow icon_pin_alt"></i> <?php esc_html_e($aEventSettings['place_detail']); ?>
                            </p>
                            <?php do_action('wiloke-listgo-functionality/public/event-list/information', $aEventSettings); ?>
                        </td>
                        <td class="listing-event__from" title="<?php esc_html_e('Events starts', 'wiloke'); ?>">
                            <p>
							    <?php if ( !empty($aEventSettings['start_at']) ) : ?>
                                    <i class="color-green icon_clock_alt"></i> <?php echo esc_html(apply_filters('wiloke/listgo/admin/public/template/vc/wiloke_events/start_at', $aEventSettings['start_at'])); ?> <br>
							    <?php endif; ?>
							    <?php if ( !empty($aEventSettings['start_on']) ) : ?>
                                    <i class="color-green icon_table"></i> <?php echo esc_html($this->renderDate($aEventSettings['start_on'])); ?>
							    <?php endif; ?>
                            </p>
                        </td>
                        <td class="listing-event__to"  title="<?php esc_html_e('Events closes', 'wiloke'); ?>">
                            <p>
							    <?php if ( !empty($aEventSettings['end_at']) ) : ?>
                                    <i class="color-red icon_clock_alt"></i> <?php echo esc_html(apply_filters('wiloke/listgo/admin/public/template/vc/wiloke_events/end_at', date($timeFormat, strtotime($aEventSettings['end_at'])))); ?> <br>
							    <?php endif; ?>
							    <?php if ( !empty($aEventSettings['end_on']) ) : ?>
                                    <i class="color-red icon_table"></i> <?php echo esc_html($this->renderDate($aEventSettings['end_on'])); ?>
							    <?php endif; ?>
                            </p>
                        </td>
                    </tr>
                </table>
            </div>

            <div class="listing-single__event-content">
			    <?php the_content(); ?>
            </div>

            <?php if ( current_user_can('edit_theme_options') || (get_post_field('post_author', $post->ID) == get_current_user_id()) ) : ?>
            <div class="listing-single__event-actions">
	            <?php
	            $expiredEditing = $this->eventExpirationEdit($post->ID, 1800);
                if ( !$expiredEditing && !current_user_can('edit_theme_options') ) :
	            ?>
                <div class="listing-single__event-msg">
		            <?php esc_html_e('This event can not be edited after', 'wiloke'); ?>
                    <span class="listing-single__event-countdown-edit">
                        <span class="listing-single__event-countdown-edit-label"><?php esc_html_e('Countdown:', 'wiloke'); ?></span>
                        <span id="listgo-countdown-editing-period-<?php echo esc_attr($post->ID); ?>" class="listing-single__event-countdown-edit-time" data-created="<?php echo esc_attr(get_post_field('post_date', $post->ID)); ?>"><?php echo esc_attr('30:00'); ?></span> <?php esc_html_e(' minutes', 'wiloke'); ?>
                    </span>
                </div>
	            <?php endif; ?>
                <div class="pull-right">
                    <?php if ( !$expiredEditing ) : ?>
                    <a id="listgo-edit-event-<?php echo esc_attr($post->ID); ?>" class="listing-single__event-edit" data-id="<?php echo esc_attr($post->ID); ?>" href="#"><i class="icon_pencil"></i> <span><?php esc_html_e('Edit', 'listgo'); ?></span></a>
                    <?php endif; ?>
                    <a id="listgo-delete-event-<?php echo esc_attr($post->ID); ?>" class="listing-single__event-remove" data-id="<?php echo esc_attr($post->ID); ?>" href="#"><i class="icon_close"></i> <span><?php esc_html_e('Remove', 'listgo'); ?></span></a>
                </div>
            </div>
            <?php endif; ?>
        </div>
        <?php
    }

	public function insertAddEventTemplateToFooter(){
		global $post;
		if ( !is_singular('listing') || !$this->allowAddingEvent($post) ){
			return false;
		}

		include plugin_dir_path(__FILE__) . 'templates/add-event.php';
		include plugin_dir_path(__FILE__) . 'templates/remove-event.php';
	}
}