<?php
namespace WilokeListgoFunctionality\Middleware;


use WilokeListgoFunctionality\Framework\Routing\InterfaceMiddleware;
use WilokeListgoFunctionality\Framework\Payment\PaymentConfiguration;
use WilokeListgoFunctionality\Framework\Helpers\GetSettings;
use WilokeListgoFunctionality\Framework\Helpers\DebugStatus;

class AddListingPlanExistsMiddleware implements InterfaceMiddleware{
	public $msg;

	public function handle( array $aOptions = [] ) {
		$this->msg = esc_html__('Whoops! This plan does not exists or no longer available anymore.', 'wiloke');

		if ( !DebugStatus::status('WILOKE_TURNON_VERIFY_CLAIM') && isset($aOptions['postID']) ){
			$listingStatus = get_post_status($aOptions['postID']);
			if ( $listingStatus == 'publish' ){
				$aClaimInfo = GetSettings::getPostMeta($aOptions['postID'], 'listing_claim');
				if ( $aClaimInfo['status'] == 'claimed' ){
					return true;
				}
			}
		}

		if ( !isset($aOptions['planID']) || empty($aOptions['planID']) ){
			$this->msg = esc_html__('Whoops! Please select a plan before.', 'wiloke');
			return false;
		}

		if ( get_post_status($aOptions['planID']) != 'publish' ){
			$this->msg = esc_html__('Whoops! This plan is no longer available.', 'wiloke');
			return false;
		}

		if ( current_user_can('administrator') ){
			return true;
		}

		if ( isset($aOptions['postID']) && !empty($aOptions['postID']) ){
			return true;
		}

		$aWilokeSubmissionSettings = PaymentConfiguration::get();
		if ( !isset($aWilokeSubmissionSettings['customer_plans']) || empty($aWilokeSubmissionSettings['customer_plans']) ){
			return false;
		}

		$aListingPlans = explode(',', $aWilokeSubmissionSettings['customer_plans']);

		if ( !in_array($aOptions['planID'], $aListingPlans) ){
			return false;
		}

		return true;
	}
}