<?php

namespace WilokeListgoFunctionality\Register;


use WilokeListgoFunctionality\Framework\Helpers\General;

class RegisterAddListingMetaBox {
	public function __construct() {
		add_action('add_meta_boxes', array($this, 'register'));
		add_action('admin_enqueue_scripts', array($this, 'enqueueScripts'));
		add_action('wp_ajax_wiloke_listgo_fetch_product_id', array($this, 'fetchProductIDs'));
		add_action('save_post_pricing', array($this, 'saveWooCommerceAssociate'), 10);
		add_action('save_post_event-pricing', array($this, 'saveWooCommerceAssociate'), 10);
	}

	public function saveWooCommerceAssociate($postID){
		if ( !current_user_can('edit_theme_options') || !isset($_POST['woocommerce_associate_id']) ){
			return false;
		}

		$previousValue = get_post_meta($postID, 'woocommerce_associate_id', true);
		if ( !empty($previousValue) ){
		    delete_post_meta($_POST['woocommerce_associate_id'], 'wiloke_woocommerce_is_associate');
        }
		update_post_meta($postID, 'woocommerce_associate_id', sanitize_text_field($_POST['woocommerce_associate_id']));

		update_post_meta($_POST['woocommerce_associate_id'], 'wiloke_woocommerce_is_associate', 'yes');
	}

	public function fetchProductIDs(){
		$aNotFound = array(
			'name' => 'Please enter your keyword',
			'value'=> ''
		);

		if ( empty($_POST['term']) ){
			wp_send_json_error(
				$aNotFound
			);
		}

		$query = new \WP_Query(
			array(
				'post_type' => 'product',
				'posts_per_page' => 100,
				'post_status' => 'publish',
				's' => $_POST['term']
			)
		);

		if ( $query->have_posts() ){
			$aProducts = array();
			while ($query->have_posts()){
				$query->the_post();
				$aData['value'] = $query->post->post_title;
				$aData['id'] = $query->post->ID;

				$aProducts[] = $aData;
			}

			wp_reset_postdata();
			wp_send_json_success($aProducts);
		}else{
			wp_send_json_error(
				$aNotFound
			);
		}
	}

	public function enqueueScripts(){
	    $postType = General::detectPostType();

		if ($postType  != 'pricing' && $postType != 'event-pricing'){
			return false;
		}
		wp_enqueue_script('jquery-ui-autocomplete');

		if ( function_exists('is_woocommerce') ){
			wp_enqueue_style('wiloke-auto-complete', plugin_dir_url(__FILE__) . '../../admin/css/wiloke-auto-complete.css');
			wp_enqueue_script('wiloke-auto-complete', plugin_dir_url(__FILE__) . '../../admin/js/wiloke-auto-complete.js', array('jquery'), WILOKE_LISTGO_FC_VERSION, true);
        }
	}

	public function register(){
		add_meta_box('woocommerce_associate', 'WooCommerce Association', array($this, 'settings'), array('pricing', 'event-pricing'));
	}

	public function settings($post){
		$wooAssociateID = get_post_meta($post->ID, 'woocommerce_associate_id', true);
		$title = !empty($wooAssociateID) ? get_the_title($wooAssociateID) : '';
		?>
		<div class="wiloke-auto-complete-wrapper">
			<label for="woocommerce_associate"><strong><?php echo 'Product name'; ?></strong></label>
			<input name="woocommerce_associate" id="woocommerce_associate" class="wiloke_auto_complete regular-text" value="<?php echo esc_attr($title); ?>">
            <img class="wiloke_auto_complete_loading hidden" src="<?php echo esc_url(WILOKE_LISTGO_FUNC_URL . 'admin/css/img/loading.gif'); ?>">
			<input type="hidden" name="woocommerce_associate_id" id="woocommerce_associate_id" class="wiloke_auto_complete_id" value="<?php echo esc_attr($wooAssociateID); ?>">
		</div>
		<?php
	}
}