;(function ($) {
	'use strict';
	
	function scrollTo($target) {
		if ( !$target.length ){
			return false;
		}

		$('html, body').animate({
			scrollTop: $target.offset().top - 100
		}, 500);
	}
	
	/*
	 |--------------------------------------------------------------------------
	 | Submit Listing
	 |--------------------------------------------------------------------------
	 | @since 1.0
	 |
	 */
	class WilokeSubmitListing{
		constructor(){
			this.$form = $('#wiloke-form-preview-listing');
			this.$submit = $('#wiloke-listgo-submit-listing');
			this.$editListing = $('#wiloke-listgo-update-listing');
			this.$acfWrapper = this.$form.find('.acf_postbox').length ? this.$form.find('.acf_postbox') : this.$form.find('.acf-fields');
			this.acfData = '';
			this.xhrPreview = null;
			this.previewListing();
			this.submitListing();
			this.removeGallery();
			this.mediaUpload();
			this.anonymousUploadFeaturedImg();
			this.anonymousUploadGalleryImg();
			this.anonymousACFUpload();
			this.toggleBusinessHours();
			this.editListing();
		}

		solvedAFCPostID(){
			$(window).on('load', (()=>{
				let $afcPostID = this.$form.find('#_acf_post_id'),
					$listingID = this.$form.find('[name="listing_id"]');
				if ( $afcPostID.length ){
					$afcPostID.val($listingID.val());
					acf.o.post_id = $listingID.val();
				}
			}))
		}

		toggleBusinessHours(){
			let $bsHours = $('#wiloke-toggle-business-hours'),
				$tblBSH = $('#wiloke-tbl-business-hours');

			$bsHours.on('change', function () {
				if ( $(this).val() === 'enable' ){
					$tblBSH.fadeIn('slow');
				}else{
					$tblBSH.fadeOut('slow');
				}
			}).trigger('change');
		}

		getContent(){
			if ($("#wp-listing_content-wrap").hasClass("tmce-active")){
				return tinyMCE.get('listing_content').getContent();
			}else{
				return $('#listing_content').val();
			}
		}

		anonymousACFUpload(){
			let xhr = null, self = this;
			$('body').on('processAFCUploads', function () {
				let $imgs = self.$form.find('.acf-field-image'), totalImgFields = $imgs.length, loopAt = 1;

				$imgs.each(function () {
					let $this = $(this);
					let keyName = $this.data('key');
					$this.wrap('<form id="wiloke-create-form-submission" enctype="multipart/form-data"></form>');
					let temporaryForm = document.getElementById('wiloke-create-form-submission'),
						oTemporaryFormData = new FormData(temporaryForm);

					xhr = $.ajax({
						url: WILOKE_GLOBAL.ajaxurl + '?type=afc_single&action=wiloke_submission_insert_media&where=acf-form-data&name='+keyName,
						type: 'POST',
						async: true,
						cache: false,
						contentType: false,
						processData: false,
						data: oTemporaryFormData,
						success: (response=>{
							if ( response.success ){
								$this.find('input[type="hidden"]').val(response.data.message);
								$this.find('[name="acf['+keyName+']"]:last').remove();
							}
							$this.unwrap();

							if ( totalImgFields = loopAt ){
								$('body').trigger('processUploadGallery');
							}

						})
					});

					loopAt++;
				})
			});
		}

		anonymousUploadFeaturedImg(){
			let xhr = null, $body = $('body');
			$body.on('processUploadFeaturedImage', function () {
				let $featuredImg = $('[name="wiloke_raw_featured_image"]');

				$featuredImg.wrap('<form id="wiloke-create-form-submission" enctype="multipart/form-data"></form>');
				let temporaryForm = document.getElementById('wiloke-create-form-submission'),
					oTemporaryFormData = new FormData(temporaryForm);

				xhr = $.ajax({
					url: WILOKE_GLOBAL.ajaxurl + '?type=single&action=wiloke_submission_insert_media&security='+WILOKE_GLOBAL.wiloke_nonce+'&where=wiloke-upload-feature-image&name=wiloke_raw_featured_image',
					type: 'POST',
					async: true,
					cache: false,
					contentType: false,
					processData: false,
					data: oTemporaryFormData,
					success: (response=>{
						if ( response.success ){
							$('#wiloke_feature_image').val(response.data.message);
						}
						$featuredImg.unwrap();
						console.log('dadad' + $('.acf-field-image').length);
						if ( $('.acf-field-image').length ){
							$body.trigger('processAFCUploads');
						}else{
							$body.trigger('processUploadGallery');
						}
					})
				})
			});
		}

		anonymousUploadGalleryImg(){
			let $body = $('body');
			let xhr = null;
			$body.on('processUploadGallery', function () {
				let $galleryImage = $('#wiloke-upload-gallery-image');

				if ( !$galleryImage.length ){
					$body.data('Wiloke/Submission/UploadedGallery', true);
					$body.trigger('galleryHasbeenUploaded');
					return false;
				}

				$galleryImage.wrap('<form id="wiloke-create-form-submission" enctype="multipart/form-data"></form>');

				let temporaryForm = document.getElementById('wiloke-create-form-submission');
				let	oTemporaryFormData = new FormData(temporaryForm);

				xhr = $.ajax({
					url: WILOKE_GLOBAL.ajaxurl + '?type=multiple&action=wiloke_submission_insert_media&security='+WILOKE_GLOBAL.wiloke_nonce+'&where=wiloke-upload-gallery-image&name=wiloke_raw_gallery_image',
					type: 'POST',
					async: true,
					cache: false,
					contentType: false,
					processData: false,
					data: oTemporaryFormData,
					success: (response=>{
						if ( response.success ){
							$('#wiloke_submission_listing_gallery').val(response.data.message);
						}
						$galleryImage.unwrap();
						$body.data('Wiloke/Submission/UploadedGallery', true);
						$body.trigger('galleryHasbeenUploaded');
					})
				})
			});
		}

		mediaUpload(){
			if ( !_.isUndefined(this.$form.data('isuserloggedin')) && this.$form.data('isuserloggedin') ){
				this.$form.find('.wiloke-add-featured-image').each(function () {
					new wilokeMediaUpload($(this));
				});
			}else{
				let $form = this.$form;
				this.$form.find('.wiloke-simple-upload').each(function () {
					new WilokeSimpleUpload($(this), $form);
				});
			}
		}

		validate(){
			let $title = $('#listing_title');
			if ( $title.val() === '' ){
				this.$error = $title;
				this.scrollTop();
				return false;
			}

			if ( this.getContent() === '' ){
				this.$error = $('#wp-listing_content-wrap');
				this.scrollTop();
				return false;
			}

			let $listingLocation = $('#listing_location');
			if ( $listingLocation.hasClass('add_listing_location_by_default') && $listingLocation.val() === '' ){
				this.$error = $listingLocation;
				this.scrollTop();
				return false;
			}

			let $listingAddress = $('#wiloke-location');
			if ( $listingAddress.val() === '' ){
				this.$error = $listingAddress;
				this.scrollTop();
				return false;
			}

			let $listingLatLong = $('#wiloke-latlong');
			if ( $listingLatLong.val() === '' ){
				this.$error = $listingLatLong;
				this.scrollTop();
				return false;
			}

			let $listingCats = $('#listing_cats');
			if ( $listingCats.val() === '' ){
				this.$error = $listingCats;
				this.scrollTop();
				return false;
			}

			if ( $('#createaccount').is(':checked') ){
				let $regEmail = $('#wiloke-reg-email');
				if ( $regEmail.val() === '' ){
					this.$error = $regEmail;
					this.scrollTop();
					return false;
				}
			}else{
				let $userName = $('#wiloke-user-login'),
					$password = $('#wiloke-my-password');
				if ( $userName.val() === '' ){
					this.$error = $userName;
					this.scrollTop();
					return false;
				}

				if ( $password.val() === '' ){
					this.$error = $password;
					this.scrollTop();
					return false;
				}
			}

			return true;
		}

		validateACF(){
			if ( this.$acfWrapper.length ){
				let fieldType = '', self = this;
				this.$acfWrapper.find('.field').each(function () {
					let $this = $(this);

					if ( $this.hasClass('required') ){
						fieldType = $this.data('field_type');
						if ( $this.find('.'+fieldType).val() === '' ){
							$this.addClass('validate-required');
							self.$error = $this;
							self.scrollTop();
							return false;
						}
					}
				})
			}
			this.getACFCustomFieldData();
			return true;
		}

		getACFCustomFieldData(){
			if ( this.$acfWrapper.length ){
				let fieldType = '', fieldName = '', oVal = {}, fieldKey = '', target = '.field',
					fieldTypeKey = 'field_type', fieldNameKey = 'field_key', fieldKeyKey = 'field_key';

				if ( this.$acfWrapper.find('.acf-field').length ){
					fieldTypeKey = 'type';
					fieldNameKey = 'name';
					fieldKeyKey  = 'key';
					target  = '.acf-field';
				}

				this.$acfWrapper.find(target).each(function () {
					let $this = $(this);
					fieldType = $this.data(fieldTypeKey);
					fieldKey  = $this.data(fieldKeyKey);
					fieldName = $this.data(fieldNameKey);

					if ( fieldType === 'wysiwyg' ){
						let editorID = $this.find('.wp-editor-area').attr('id');
						if ($("#wp-acf_settings-wrap").hasClass("tmce-active")){
							oVal[fieldKey] = tinyMCE.get(editorID).getContent();
						}else{
							oVal[fieldKey] = $('#'+editorID).val();
						}
					}else if(fieldType === 'checkbox'){
						$this.find('[type="checkbox"]:checked').each(function () {
							if ( typeof oVal[fieldKey] === 'undefined' ){
								oVal[fieldKey] = [];
							}
							oVal[fieldKey].push($(this).val());
						})
					}else{
						let $fieldVal = $this.find('[name="fields['+fieldKey+']"]').length ? $this.find('[name="fields['+fieldKey+']"]') : $this.find('[name="acf['+fieldKey+']"]');
						oVal[fieldKey] = $fieldVal.val();
					}
				});

				this.acfData = JSON.stringify(oVal);
			}
		}

		scrollTop(){
			if ( !this.$error.length ){
				return false;
			}

			this.addErrorClass();

			$('html, body').animate({
				scrollTop: this.$error.offset().top - 40
			}, 300);
		}

		addErrorClass(){
			this.$error.closest('.form-item').addClass('validate-required');
		}

		removeGallery(){
			this.$form.on('click', '.wil-addlisting-gallery__list-remove', function () {
				if ( WILOKE_GLOBAL.isLoggedIn === 'no' ){
					$(this).closest('#wiloke-show-gallery').empty();
				}else{
					$(this).closest('.gallery-item').remove();
				}
			});
		}

		requestGallery(){
			let $galleryID = this.$form.find('#wiloke-preview-gallery'),
				galleryIDs = [];

			if ( $galleryID.length ){
				$galleryID.find('.gallery-item').each(function () {
					if ( typeof $(this).data('id') !=='undefined' ){
						galleryIDs.push($(this).data('id'));
					}
				});

				let convertGalleryToString = galleryIDs.join(',');
				this.$form.find('#listing_gallery').val(convertGalleryToString);
			}
		}

		validateEmailRegister(){
			$.ajax({
				url: WILOKE_GLOBAL.ajaxurl,
				type: 'POST',
				data: {
					action: 'wiloke_verify_email',
					email: $('#wiloke-reg-email').val(),
					security: WILOKE_GLOBAL.wiloke_nonce
				},
				success: (response=>{
					if ( response.success ){
						$('body').data('WilokeUserLoggedIn', true);
						this.$form.find('#wiloke-reg-invalid-email').addClass('hidden');
						this.$form.trigger('submit');
					}else{
						this.$form.find('#wiloke-reg-invalid-email').removeClass('hidden');
						this.$form.trigger('stopping_upload');
					}
				})
			});
		}

		previewListing(){
			if ( this.$form.length ){
				let $body = $('body');
				this.$error = null;
				this.$preview = $('#wiloke-listgo-preview-listing');
				this.$preview.addClass('loading');
				this.$printMessage = this.$form.find('.wiloke-print-msg-here');

				$body.on('galleryHasbeenUploaded', (()=>{
					this.$form.trigger('submit');
				}));

				this.$form.on('submit', ((event)=>{
					event.preventDefault();
					let $target = $(event.target);
					this.ajaxPreviewListing($target);
				}));
			}
		}

		submitListing(){
			if ( this.$submit.length ){
				this.$submit.removeClass('not-active');
				this.$submit.on('click', (event=>{
					event.preventDefault();
					let $target = $(event.currentTarget);

					if ( $target.data('processing') ){
						return false;
					}

					$target.addClass('loading');
					$target.data('processing', true);

					this.requestGallery();

					$.ajax({
						type: 'POST',
						global: false,
						url: WILOKE_GLOBAL.ajaxurl,
						data: {action: 'wiloke_submission_submit_listing', 'security': WILOKE_GLOBAL.wiloke_nonce, post_id: $target.data('postid'), planID: $('#package_id').val()},
						success: (response=>{
							if ( response.success ){
								window.location.href = decodeURIComponent(response.data.redirectTo);
							}else{
								alert(response.data.msg);
							}

							$target.removeClass('loading');
							$target.data('processing', false);
						})
					})
				}))
			}
		}

		ajaxPreviewListing($target){
			let $body = $('body');
			let isValidated = this.validate();
			this.$printMessage.html('');
			if ( !isValidated ){
				return false;
			}

			isValidated = this.validateACF();

			if ( !isValidated ){
				return false;
			}

			this.$preview.prop('disabled', true);
			this.$preview.addClass('loading');

			if ( this.xhrPreview !== null && this.xhrPreview.status !== 200 ){
				this.$preview.removeClass('loading');
				this.$preview.prop('disabled', false);
				this.xhrPreview.abort();
			}

			this.$form.on('stopping_upload', (event=>{
				this.$preview.removeClass('loading');
				this.$preview.prop('disabled', false);
				if ( this.xhrPreview !== null && this.xhrPreview !== 200 ){
					this.xhrPreview.abort();
				}
				return false;
			}));

			this.$preview.addClass('loading');
			if ( !$body.data('WilokeUserLoggedIn') && WILOKE_GLOBAL.isLoggedIn === 'no' && $('#createaccount').is(':checked') ) {
				this.validateEmailRegister();
				return false;
			}

			if ( (WILOKE_GLOBAL.isLoggedIn === 'no') && !$body.data('Wiloke/Submission/UploadedGallery') ){
				$body.trigger('processUploadFeaturedImage');
				return false;
			}

			let content = this.getContent();

			this.requestGallery();

			this.xhrPreview = $.ajax({
				type: 'POST',
				url: WILOKE_GLOBAL.ajaxurl + '?action=wiloke_submission_new_preview_listing',
				data: {security: WILOKE_GLOBAL.wiloke_nonce, data: $target.serialize(), content: content, acfData: this.acfData},
				success: (response=>{
					if ( response.success ){
						window.location.href = response.data.next;
					}else{
						let messages = '';
						_.forEach(response.data, ((message, id)=>{
							let $target = $('#'+id);
							if ($target.length ){
								$target.html(message).removeClass('hidden');
								$target.parent().removeClass('hidden').addClass('validate-required');
							}
							messages += '<p class="update-status error-msg">'+message+'</p>';
						}));

						if ( messages !== '' ){
							this.$printMessage.html(messages).removeClass('hidden');
						}

						if ( response.data.isCreatedNewAccount ){
							setTimeout(function(){
								location.reload();
							}, 2000);
						}
					}

					this.$preview.prop('disabled', false);
					this.$preview.removeClass('loading');
				})
			})
		}

		ajaxUpdateListing(){
			let isValidated = this.validate();
			this.$printMessage.html('');
			if ( !isValidated ){
				return false;
			}

			isValidated = this.validateACF();

			if ( !isValidated ){
				return false;
			}


			if ( this.xhrUpdateListing !== null && this.xhrUpdateListing.status !== 200 ){
				this.xhrUpdateListing.abort();
			}

			this.$form.addClass('loading');

			this.ajaxPreviewListing(this.$form);
		}

		confirmEditListing($popup){
			$('#listgo-cancel-edit-listing').on('click', (event=>{
				event.preventDefault();
				this.$editListing.removeClass('loading');
				$popup.removeClass('wil-modal--open');
			}));

			$('#listgo-continue-editing-listing').on('click', (event=>{
				this.$form.trigger('submit');
				event.preventDefault();
				$popup.removeClass('wil-modal--open');
			}));
		}

		editListing(){
			if ( this.$editListing.length ){
				this.xhrUpdateListing = null;
				let $popup = $('#wiloke-form-update-listing-wrapper');
				this.$editListing.on('click', (event=>{
					event.preventDefault();
					this.$editListing.addClass('loading');

					let $target = $(event.preventDefault);
					if ( this.$editListing.data('edittype') === 'allow_need_review' ){
						$popup.addClass('wil-modal--open');
						this.confirmEditListing($popup);
					}else{
						this.$form.trigger('submit');
					}

				}))
			}
		}
	}

	class wilokeMediaUpload{
		constructor($target){
			this.$trigger = $target;
			this.init();
		}

		template(url, id){
			let item = _.template("<li class='gallery-item bg-scroll' data-id='<%- id %>' style='background-image: url(<%- backgroundUrl %>)'><span class='wil-addlisting-gallery__list-remove'>Remove</span></li>");
			return item({
				backgroundUrl: url,
				id: id
			});
		}

		init(){
			if ( this.$trigger.length ){

				// ADD IMAGE LINK
				this.$trigger.on( 'click', (event=>{
					let $target = $(event.currentTarget);
					event.preventDefault();

					let isMultiple = $target.data('multiple');
					// If the media frame already exists, reopen it.
					if ( $target.data('frame') ) {
						$target.data('frame').open();
						return false;
					}

					// Create a new media frame
					let frame = wp.media({
						title: '',
						button: {
							text: 'Select'
						},
						multiple: isMultiple  // Set to true to allow multiple files to be selected
					});
					$target.data('frame', frame);
					let imgSize = typeof $target.data('imgsize') !== 'undefined' ? $target.data('imgsize') : 'thumbnail';

					// When an image is selected in the media frame...
					$target.data('frame').on( 'select', (()=>{
						// Get media attachment details from the frame state

						if ( isMultiple ){
							let aAttachemnts = $target.data('frame').state().get('selection').toJSON();
							let imgs = '';
							_.forEach(aAttachemnts, (oAttachment=>{
								let thumbnailUrl = !_.isUndefined(oAttachment.sizes.thumbnail) && oAttachment.sizes[imgSize] ? oAttachment.sizes[imgSize].url : oAttachment.url;
								imgs += this.template(thumbnailUrl, oAttachment.id);
							}));
							$target.parent().before(imgs);
						}else{
							let attachment = $target.data('frame').state().get('selection').first().toJSON();
							// Send the attachment URL to our custom image input field.
							let thumbnailUrl = !_.isUndefined(attachment.sizes[imgSize]) && attachment.sizes[imgSize] ? attachment.sizes[imgSize].url : attachment.url;

							$target.find('.wiloke-preview').attr( 'src', thumbnailUrl );
							$target.find('.add-listing__upload-preview').css( 'background-image', 'url('+attachment.url+')' );
							// Send the attachment id to our hidden input
							$target.find('.wiloke-insert-id').val( attachment.id );
						}

					}));

					// Finally, open the modal on click
					$target.data('frame').open();
				}));
			}
		}
	}

	class WilokeSimpleUpload{
		constructor($target, $form){
			this.$target = $target;
			this.$form = $form;
			this.$wrapper = this.$target.closest('.upload-file');
			this.maxfilesize = $form.data('uploadfilesize');
			this.xhr = null;
			this.uploadInputChanged();
			this.removeSingleUpload();
			this.fileStore = [];
		}

		uploadInputChanged(){
			let self = this;
			this.$remider = this.$target.closest('.input-upload-file').siblings('.wiloke-submission-reminder');
			if ( isNaN(this.maxfilesize) ){
				this.maxfilesize = parseInt(this.maxfilesize.replace('M', ''), 10)*1000*1000;
			}

			this.$target.on('change', function(){
				self.aListFiles = this.files;
				self.showImageReview();
			});
		}

		removeSingleUpload(){
			this.$form.on('click', '.wil-addlisting-gallery__list-remove', (event=>{
				if ( $(event.currentTarget).closest('.wil-addlisting-gallery').hasClass('single-upload') ){
					$('#wiloke-show-featured-image').find('.wil-addlisting-gallery__list').empty();
				}else{
					$(event.currentTarget).parent().remove();
				}
			}));
		}

		showImageReview(){
			if ( this.aListFiles.length ){
				let $previewImgs= this.$target.hasClass('wiloke_feature_image') ? $('#wiloke-show-featured-image').find('.wil-addlisting-gallery__list') : $('#wiloke-show-gallery'),
					isMultiple  = !_.isUndefined(this.$target.data('ismultiple')),
					$remider    = this.$remider,
					maxFileSize = this.maxfilesize;

				_.each(this.aListFiles, (file=>{
					if ( /\.(jpe?g|png|gif)$/i.test(file.name) ) {

						let reader = new FileReader();
						reader.addEventListener('load', function () {
							let errorClass = file.size <= maxFileSize ? '' : ' error';
							let img = '<li class="bg-scroll'+errorClass+'" style="background-image: url('+this.result+')"><span class="wil-addlisting-gallery__list-remove">Remove</span></li>';
							
							if ( isMultiple ){
								$previewImgs.append(img);
							}else{
								$previewImgs.html(img);
							}
							$remider.removeClass('review_status error-msg');

							$('body').trigger('processUpload');

						}, false);

						reader.readAsDataURL(file);
					}
				}));
			}
		}
	}

	$(document).ready(function () {
		$('.input-select2').children().each(function () {
			let oConfiguration = {};
			let $this = $(this);
			if( $this.data('maximumSelection') ){
				oConfiguration.maximumSelectionLength = $this.data('maximumSelection');
			}

			if( $this.data('placeholder') ){
				oConfiguration.placeholder = $this.data('placeholder');
			}

			$this.select2(oConfiguration);
		});
	});

	$(window).load(function () {
		new WilokeSubmitListing();
	})
})(jQuery);