;(function ($) {
	'use strict';

	class wilokeMediaUpload{
		constructor(){
			this.$trigger = $('.wiloke-js-upload');
			this.init();
		}

		init(){
			if ( this.$trigger.length ){
				// ADD IMAGE LINK
				this.$trigger.on( 'click', (event=>{
					let $target = $(event.currentTarget);
					event.preventDefault();

					// If the media frame already exists, reopen it.
					if ( $target.data('frame') ) {
						$target.data('frame').open();
						return false;
					}

					// Create a new media frame
					let frame = wp.media({
						title: 'Select or Upload Media',
						button: {
							text: 'Use this media'
						},
						multiple: false  // Set to true to allow multiple files to be selected
					});
					$target.data('frame', frame);

					// When an image is selected in the media frame...
					$target.data('frame').on( 'select', (()=>{
						// Get media attachment details from the frame state
						let attachment = $target.data('frame').state().get('selection').first().toJSON();
						// Send the attachment URL to our custom image input field.
						$target.find('.wiloke-preview').attr( 'src', attachment.url ).removeClass('hidden');
						$('#wiloke-avatar-by-text').addClass('hidden');
						// Send the attachment id to our hidden input
						$target.find('.wiloke-insert-id').val( attachment.id );

						if( $target.hasClass('profile-background') ) {
							$target.css('background-image', 'url('+ attachment.url +')')
						}

						$('.header-page__account-avatar-img').html('<img src="'+attachment.url+'" class="avatar">');
					}));

					// Finally, open the modal on click
					$target.data('frame').open();
				}));
			}
		}
	}

	class wilokeUpdatePlan{
		constructor(){
			this.$app = $('#wiloke-modify-subscription-plan-form');
			this.$controlPlan = $('#wiloke-submission-customer-plan');
			this.init();
		}
		init(){
			if ( !this.$app.length  ){
				return false;
			}

			this.proceedAjax = false;
			this.$confirmPopup = $('#wiloke-confirm-change-plan-wrapper');
			this.$wantToChangeBtn = $('#wiloke-want-to-change-plan');
			this.$cancelChangeBtn = $('#wiloke-cancel-change-plan');
			this.$proceedWith = $('#wiloke-submission-proceed-with');
			this.gateway = null;
			this.token = null;

			if ( typeof TCO !== 'undefined' ){
				TCO.loadPubKey('sandbox');
			}
			

			this.listenGateway();
			this.confirmChangePlan();
			this.cancelChangePlan();
			this.previewPlan();

			this.$btn = this.$app.find('button');
			this.$ajax = null;
			this.$successMsg = $('#wiloke-success-change-plan');
			this.$failedMsg = $('#wiloke-failed-change-plan');

			this.$app.on('submit', (event=>{
				event.preventDefault();

				if ( !this.proceedAjax ){
					this.$confirmPopup.addClass('wil-modal--open');
					return false;
				}

				if ( this.$ajax !== null && this.$app.status !== 200 ){
					this.$ajax.abort();
				}

				this.$btn.addClass('loading');

				if ( this.gateway !== '2checkout' ){
					this.ajaxRequest();
				}else{
					if ( this.checkCardForm() ){
						this.twoCheckoutTokenRequest();
					}else{
						$(window).scrollTop($('#wiloke-my-credi-debit-card-form').offset().top);
						this.$btn.removeClass('loading');
					}
				}

			}));
		}

		checkCardForm(){
			let passed = true;
			$('#wiloke-my-credi-debit-card-form').find('.row input').each(function(){
				if ( $(this).attr('name') !== '' ){
					if ( $(this).val() === '' ){
						$(this).closest('.form-item').addClass('validate-required');
						passed = false;
					}
				}
			});

			return passed;
		}

		previewPlan(){
			let xhr = null;
			let aCachingPlan = [];
			let $previewArea = $("#wiloke-show-package-detail");
			this.$controlPlan.change((event=>{
				if ( xhr !== null && xhr.status !== 200 ){
					xhr.abort();
				}
				let $target = $(event.target);
				$previewArea.addClass('loading');
				let packageID = $target.val();

				if ( !_.isUndefined(aCachingPlan[packageID]) ){
					$previewArea.html(aCachingPlan[packageID]);
					$previewArea.removeClass('loading');
					return false;
				}

				xhr = $.ajax({
					type: 'GET',
					url: WILOKE_GLOBAL.ajaxurl,
					data: {action: 'wiloke_submission_render_package_preview', packageID: packageID},
					success: (response=>{
						$previewArea.html(response.data.msg);
						$previewArea.removeClass('loading');
						aCachingPlan[packageID] = response.data.msg;
					})
				})
			}));
		}

		confirmChangePlan(){
			this.$wantToChangeBtn.on('click', (event=>{
				event.preventDefault();
				this.$confirmPopup.removeClass('wil-modal--open');
				this.gateway = this.$proceedWith.val();
				this.proceedAjax = true;
				this.$app.trigger('submit');
			}))
		}

		listenGateway(){
			this.$proceedWith.change((event)=>{
				this.gateway = this.$proceedWith.val();
			})
		}

		errorCallback(data){
			if (data.errorCode === 200) {
				// This error code indicates that the ajax call failed. We recommend that you retry the token request.
			} else {
				alert(data.errorMsg);
			}
			this.$btn.removeClass('loading');
		}

		successCallback(data){
			// Set the token as the value for the token input
			this.token = data.response.token.token;
			// IMPORTANT: Here we call `submit()` on the form element directly instead of using jQuery to prevent and infinite token request loop.
			this.ajaxRequest();
		}

		twoCheckoutTokenRequest(){
			// Setup token request arguments
			let args = {
				sellerId: WILOKE_GLOBAL.twoCheckoutSellerID,
				publishableKey: WILOKE_GLOBAL.twoCheckoutPublishableKey,
				ccNo: $("#cardNumber").val(),
				cvv: $("#cvv").val(),
				expMonth: $("#expMonth").val(),
				expYear: $("#expYear").val()
			};
			// Make the token request
			TCO.requestToken((data=>{
				this.successCallback(data)
			}), (data=>{
				this.errorCallback(data)
			}), args);
		}

		cancelChangePlan(){
			this.$cancelChangeBtn.on('click', (event=>{
				event.preventDefault();
				this.proceedAjax = false;
				this.$confirmPopup.removeClass('wil-modal--open');
			}))
		}

		ajaxRequest(){
			this.$ajax = $.ajax({
				type: 'POST',
				url: WILOKE_GLOBAL.ajaxurl,
				data: {action: 'wiloke_submission_change_add_listing_plan', gateway: this.gateway, security: WILOKE_GLOBAL.wiloke_nonce, planID: this.$app.find('#wiloke-submission-customer-plan').val(), token: this.token, cardForm: $('#wiloke-my-credi-debit-card-form').serializeArray()},
				success: (response=>{
					if ( typeof response.success === 'undefined' ){
						this.$successMsg.addClass('hidden');
						this.$failedMsg.find('.wil-alert-message').html(response);
						this.$failedMsg.removeClass('hidden');
					}else{
						$('#wiloke-transaction-message').addClass('hidden');
						if ( response.success ){
							this.$failedMsg.addClass('hidden');
							this.$successMsg.find('.wil-success-message').html(response.data.msg);
							this.$successMsg.removeClass('hidden');

							if ( typeof response.data.redirectTo !== 'undefined' ){
								window.location.href = response.data.redirectTo;
							}else{
								setTimeout(function () {
									location.reload(true);
								}, 4000);
							}
						}else{
							this.$successMsg.addClass('hidden');
							this.$failedMsg.find('.wil-alert-message').html(response.data.msg);
							this.$failedMsg.removeClass('hidden');
						}
					}
					this.proceedAjax = false;
					this.$btn.removeClass('loading');
				})
			})
		}
	}

	class wilokeBillOutstandingAmount{
		constructor(){
			this.$app = $('#wiloke-billing-outstanding-amount-form');
			this.init();
		}

		init(){
			this.xhr = null;
			if ( this.$app.length > 0 ){
				let $btn = this.$app.find('button'),
					$successMsg = $('#wiloke-success-bill-outstanding'),
					$failedMsg = $('#wiloke-failed-bill-outstanding');
				this.$app.on('click', (event=>{
					event.preventDefault();
					$btn.addClass('loading');
					if ( this.xhr !== null && this.xhr.status !== 200 ){
						this.xhr.abort();
					}

					this.xhr = $.ajax({
						type: 'POST',
						url: WILOKE_GLOBAL.ajaxurl,
						data: {action: 'wiloke_paypal_bill_outstanding_amount', security: WILOKE_GLOBAL.wiloke_nonce},
						success: (response=>{
							if ( response.success ){
								$successMsg.find('.wil-alert-message').html(response.data.msg);
								$successMsg.removeClass('hidden');
								$failedMsg.addClass('hidden');
								this.$app.remove();
							}else{
								$failedMsg.find('.wil-alert-message').html(response.data.msg);
								$failedMsg.removeClass('hidden');
								$successMsg.addClass('hidden');
							}
							$btn.removeClass('loading');
						})
					})
				}))
			}
		}
	}

	class wilokeRenderPlanMessage{
		constructor(){
			this.$msg = $('#wiloke-transaction-message');
			this.message();
		}

		message(){
			if ( this.$msg.length ){
				this.$form = this.$msg.closest('#wiloke-modify-subscription-plan-form');
				this.$form.addClass('loading');
				$.ajax({
					type: 'GET',
					url: WILOKE_GLOBAL.ajaxurl,
					data: {security: WILOKE_GLOBAL.wiloke_nonce, action: 'wiloke_change_plan_message', token: this.$msg.data('token')},
					success: (response=>{
						if ( typeof response.data.msg !== 'undefined' ){
							if ( response.success ){
								this.$msg.attr('class', 'wil-alert wil-alert-has-icon alert-success');
								this.$msg.html('<span class="wil-alert-icon"><i class="icon_box-checked"></i></span><p class="wil-alert-message">'+response.data.msg+'</p>');
								let $packageSelected = $('#wiloke-submission-customer-plan').find('option[value="'+response.data.packageID+'"]');
								$packageSelected.attr('selected', 'selected');
							}else{
								this.$msg.attr('class', 'wil-alert wil-alert-has-icon alert-danger');
								this.$msg.html('<span class="wil-alert-icon"><i class="icon_error-triangle_alt"></i></span><p class="wil-alert-message">'+response.data.msg+'</p>');
							}
						}else{
							this.$msg.addClass('hidden');
						}

						this.$form.removeClass('loading');
					})
				})
			}
		}
	}

	class wilokeSaveCard{
		constructor(){
			this.$app = $("#wiloke-my-credi-debit-card-form");
			this.init();
		}

		init(){
			if ( this.$app.length > 0 ){
				this.xhr = null;
				this.$msg = $("#wiloke-save-card-msg-wrapper");
				this.$app.on('submit', (event=>{
					event.preventDefault();

					if ( this.xhr !== null && this.xhr.status !== 200 ){
						this.xhr.abort();
					}
					this.$app.addClass('loading');
					this.xhr = $.ajax({
						type: 'POST',
						url: WILOKE_GLOBAL.ajaxurl,
						data: {action: 'wiloke_save_card', data: this.$app.serializeArray()},
						success: (response=>{
							this.$msg.find('.wil-alert-message').html(response.data.msg);
							this.$msg.removeClass('hidden');
							this.$app.removeClass('loading');
							this.$app.find('.form-item').each(function () {
								$(this).removeClass('validate-required');
							})
						})
					})
				}))
			}
		}
	}

	function cancelPlan(){
		$('.listgo-cancel-plan').on('click', function(event){
			event.preventDefault();
			let iWantToDoIt = confirm(WILOKE_LISTGO_TRANSLATION.confirmCancelPlan);
			if ( !iWantToDoIt ){
				return false;
			}

			let $this = $(this);
			let sessionID 	= $(this).data('sessionid');
			let gateway 	= $(this).data('gateway');

			$this.addClass('loading');

			jQuery.ajax({
				type: 'POST',
				url: WILOKE_GLOBAL.ajaxurl,
				data: {
					action: 'listgo_cancel_plan_'+gateway,
					sessionID: sessionID,
					gateway: gateway	
				},
				success: function(response){
					$this.removeClass('loading');
					if ( typeof response.data.msg !== 'undefined' ){
						alert(response.data.msg);
					}else{
						alert(response.data);
					}
					if ( response.success ){
						location.reload();
					}
				}
			})
		})
	}

	function colorPicker(){
		$('.colorpicker').each(function () {
			let $this = $(this);
			$this.spectrum({
				showAlpha: true,
				showInput: true,
				allowEmpty:true,
				change: function(color) {
					$this.val(color.toRgbString());
				}
			});
		})

	}

	$(document).ready(function () {
		new wilokeMediaUpload();
		new wilokeUpdatePlan();
		new wilokeBillOutstandingAmount();
		new wilokeSaveCard();
		cancelPlan();
		colorPicker();
	});

	$(window).load(function () {
		new wilokeRenderPlanMessage();
	})

})(jQuery);