<?php
/*
 * About Us/Me
 * @copyright Wiloke
 * @website https://wiloke.com
 */
use WilokeWidget\Supports\Helpers;

class WilokeListingInformation extends WP_Widget
{
    public $aDef = array('url' => '', 'description'=>'', 'name'=>'Wiloke');
    public function __construct()
    {
        parent::__construct('wiloke_about', WILOKE_WIDGET_PREFIX . ' (OSP) Information', array('classname'=>'widget_about widget_author widget_text', 'description'=>esc_html__('OSP - Only Single Page. This widget is only available for single listing page.', 'wiloke-listgo-widgets')) );
    }

    public function form($aInstance)
    {
        $aInstance = wp_parse_args($aInstance, $this->aDef);
	    do_action('wiloke-widgets/app/Widgets/WilokeListingInformation', $aInstance);
    }

    public function update($aNewinstance, $aOldinstance) {
        $aInstance = $aOldinstance;
        foreach ($aNewinstance as $key => $value) {
            $aInstance[$key] = $value;
        }

        return $aInstance;
    }

    public function widget($atts, $aInstance)
    {
        global $post, $wiloke;
        $aInstance = wp_parse_args($aInstance, $this->aDef);
        if ( !is_singular('listing') ){
            return false;
        }
        $aUserInfo = WilokePublic::getUserMeta($post->post_author);
        $aGeneralSettings = Wiloke::getPostMetaCaching($post->ID, 'listing_settings');
        $aSocialMedia = Wiloke::getPostMetaCaching($post->ID, 'listing_social_media');
	    $planID = Wiloke::getPostMetaCaching($post->ID, wilokeRepository('app:belongsToPlanID'));
        $aPlanSettings = Wiloke::getPostMetaCaching($planID, 'pricing_settings');
	    $aListingSettings = Wiloke::getPostMetaCaching($post->ID, 'listing_settings');

	    if ( has_action('wiloke-widgets/app/Widgets/WilokeListingInformation') ) :
		    do_action('wiloke-widgets/app/Widgets/WilokeListingInformation', $atts, $aInstance, $aUserInfo);
        else:
            echo $atts['before_widget'];
        ?>
            <div class="widget_author__header">
                <?php 
                if ( isset($aListingSettings['logo']) && !empty($aListingSettings['logo']) ) : 
                    $website = isset($aListingSettings['website']) && !empty($aListingSettings['website']) ? $aListingSettings['website'] : get_permalink($post->ID); 
                ?>
                <a href="<?php echo esc_url($website); ?>">
                    <?php if ( isset($aUserInfo['meta']['wiloke_profile_picture']) ) : ?>
                    <div class="widget_author__avatar">
                        <img src="<?php echo esc_url($aListingSettings['logo']); ?>" alt="<?php echo esc_attr($post->post_title); ?>">
                    </div>
                    <?php endif; ?>
                </a>
                <?php else: ?>
                <a href="<?php echo esc_url(get_author_posts_url($post->post_author)); ?>">
                    <?php if ( isset($aUserInfo['meta']['wiloke_profile_picture']) ) : ?>
                    <div class="widget_author__avatar">
                        <?php
                            $img = wp_get_attachment_image($aUserInfo['meta']['wiloke_profile_picture'], array(128, 128));
                            if(!empty($img)) {
                                print $img;
                            } else {
                                $firstCharacter = strtoupper(substr($aUserInfo['display_name'], 0, 1));
                                echo '<span style="background-color: '.esc_attr(WilokePublic::getColorByAnphabet($firstCharacter)).'" class="widget_author__avatar-placeholder">'. esc_html($firstCharacter) .'</span>';
                            }
                        ?>
                    </div>
                    <?php endif; ?>

                    <div class="overflow-hidden">
                        <h4 class="widget_author__name"><?php echo esc_html($aUserInfo['display_name']); ?></h4>
                        <span class="widget_author__role"><?php WilokePublic::renderBadge($aUserInfo['role']); ?></span>
                    </div>
                </a>
                <?php endif; ?>
                
            </div>

            <?php if ( $wiloke->aThemeOptions['listing_toggle_following_author'] == 'enable' ) : ?>
            <div class="account-subscribe">
                <?php
                $profileUrl = WilokePublic::getPaymentField('myaccount', true);
                $followingUrl = !empty($profileUrl) ? WilokePublic::addQueryToLink($profileUrl, 'mode=following&user='.$post->post_author) : '#';
                $followersUrl = !empty($profileUrl) ? WilokePublic::addQueryToLink($profileUrl, 'mode=followers&user='.$post->post_author) : '#';
                ?>
                <span class="followers"><a href="<?php echo esc_url($followersUrl) ?>"><span class="count"><?php echo esc_html(WilokePublic::getNumberOfFollowers($post->post_author)); ?></span> <?php esc_html_e('Followers', 'listgo'); ?></a></span>
                <span class="following"><a href="<?php echo esc_url($followingUrl) ?>"><span class="count"><?php echo esc_html(WilokePublic::getNumberOfFollowing($post->post_author)); ?></span> <?php esc_html_e('Following', 'listgo'); ?></a></span>
	            <?php
	            if ( empty(WilokePublic::$oUserInfo) || ( !empty(WilokePublic::$oUserInfo) && ($post->post_author != WilokePublic::$oUserInfo->ID) ) ) :
		            if ( $wiloke->aThemeOptions['listing_toggle_following_author'] === 'enable' ) :
			            $followStatus = esc_html__('Follow', 'wiloke-listgo-widgets');
			            if ( !empty(WilokePublic::$oUserInfo) ){
				            $status = WilokePublic::isFollowing($post);
				            if ( $status ){
					            $followStatus = esc_html__('Following', 'wiloke-listgo-widgets');
				            }
			            }
			            ?>
                        <a href="#" class="js_subscribe widget_author__follow listgo-btn listgo-btn--sm" data-authorid="<?php echo esc_attr($post->post_author); ?>"><?php echo esc_html($followStatus); ?> <i class="fa fa-rss"></i></a>
                <?php endif; endif; ?>
            </div>
            <?php endif; ?>

            <div class="widget_author__content">

                <ul class="widget_author__address">
                    <?php if ( isset($aGeneralSettings['map']['location']) && !empty($aGeneralSettings['map']['location']) ) : ?>
                    <li class="map-item">
                        <i class="fa fa-map-marker"></i>
                        <?php echo Helpers::kses_html($aGeneralSettings['map']['location']); ?>
                    </li>
                    <?php endif; ?>

                    <?php if ( isset($aGeneralSettings['contact_email']) && !empty($aGeneralSettings['contact_email']) ) : ?>
                    <li class="contact-item">
                        <i class="fa fa-envelope"></i>
                        <a href="mailto:<?php echo esc_attr($aGeneralSettings['contact_email']); ?>"><?php echo Helpers::kses_html($aGeneralSettings['contact_email']); ?></a>
                    </li>
                    <?php endif; ?>

                    <?php if ( !has_action('wiloke-listgo-widgets/listing-information/modify-phone') ) : ?>
	                <?php if ( isset($aGeneralSettings['phone_number']) && !empty($aGeneralSettings['phone_number']) ) : ?>
                    <li class="phone-number-item">
                        <i class="fa fa-phone"></i>
                        <a href="tel:<?php echo esc_attr($aGeneralSettings['phone_number']); ?>"><?php echo Helpers::kses_html($aGeneralSettings['phone_number']); ?></a>
                    </li>
	                <?php elseif ( !empty($aUserInfo['meta']['wiloke_phone']) ) : ?>
                        <li class="phone-number-item">
                            <i class="fa fa-phone"></i>
                            <a href="tel:<?php echo esc_attr($aUserInfo['meta']['wiloke_phone']); ?>"><?php echo Helpers::kses_html($aUserInfo['meta']['wiloke_phone']); ?></a>
                        </li>
	                <?php endif; ?>
                    <?php else: ?>
                        <?php echo do_action('wiloke-listgo-widgets/listing-information/modify-phone', $aGeneralSettings, $aSocialMedia, $aUserInfo); ?>
                    <?php endif; ?>

	                <?php if ( isset($aGeneralSettings['website']) && !empty($aGeneralSettings['website']) ) :  ?>
                    <li class="website-item">
                        <i class="fa fa-globe"></i>
                        <?php if ( strlen($aGeneralSettings['website']) <= 50 ) : ?>
                        <a href="<?php echo esc_url($aGeneralSettings['website']); ?>" target="_blank"><?php echo esc_html($aGeneralSettings['website']); ?></a>
                        <?php else: ?>
                            <a href="<?php echo esc_url($aGeneralSettings['website']); ?>" target="_blank"><?php echo esc_html__('Click to get more information', 'wiloke-listgo-widgets'); ?></a>
                        <?php endif; ?>
                    </li>
	                <?php elseif ( !empty($aUserInfo['user_url']) ) : ?>
                    <li class="user-url-item">
                        <i class="fa fa-globe"></i>
                        <a href="<?php echo esc_url($aUserInfo['user_url']); ?>" target="_blank"><?php echo esc_html($aUserInfo['user_url']); ?></a>
                    </li>
	                <?php endif; ?>

                    <?php do_action('wiloke-listgo-widgets/listing-information/user-info', $aUserInfo, $aGeneralSettings); ?>
                </ul>

                <?php if ( !empty($aSocialMedia) || !empty($aUserInfo['meta']['wiloke_user_socials']) ) : ?>
                <div class="widget_author__social">
                    <?php
                    $aSocialNetworks = apply_filters('listgo/filter/social-networks', WilokeSocialNetworks::$aSocialNetworks);
                    
                    foreach ( $aSocialNetworks as $socialIcon ) :
                        if ( $socialIcon == 'vk' ){
                            $socialKey = 'vkontakte';
                        }else{
                            $socialKey = $socialIcon;
                        }
                        
                        if ( \WilokeListgoFunctionality\Framework\Helpers\AddListingHelpers::isExceptThisSocial($aPlanSettings, $socialKey) ){
                            continue;
                        }

                        if ( !isset($aSocialMedia[$socialKey]) || empty($aSocialMedia[$socialKey]) ){
                            $url = isset($aUserInfo['meta']['wiloke_user_socials'][$socialKey]) ? $aUserInfo['meta']['wiloke_user_socials'][$socialKey] : '';
                        }else{
                            $url = $aSocialMedia[$socialKey];
                        }
                        if ( !empty($url) ) :
                            if ( $socialKey == 'whatsapp' ){
	                            if ( strpos($url, 'http') === false ){
		                            if ( preg_match('/[0-9]/', $url) ){
			                            $url = 'https://api.whatsapp.com/send?phone='.$url;
		                            }else{
			                            $url = 'https://api.whatsapp.com/send?text='.$url;
		                            }
	                            }
                            }else if ( $socialKey == 'wikipedia' ){
                                $socialKey = 'wikipedia-w';
                            }
	                        $socialIcon = 'fa fa-'.str_replace('_', '-', $socialIcon);
                    ?>
                        <a href="<?php echo esc_url($url); ?>" target="_blank"><i class="<?php echo esc_attr($socialIcon); ?>"></i></a>
                    <?php
                        endif;
                    endforeach;
                    ?>
                </div>
                <?php endif; ?>

                <?php
                $aCoupon = get_post_meta($post->ID, 'listing_coupon', true);
                if ( !empty($aCoupon['coupon_code']) ){
                    if ( !empty($aCoupon['referral_link']) ) :
                ?>
                <div class="text-center">
                    <a href="javascript:void(0)" onclick="s=prompt('<?php echo esc_attr($aCoupon['description']); ?>','<?php echo esc_attr($aCoupon['coupon_code']); ?>'); if (s){ window.open('<?php echo esc_attr($aCoupon['referral_link']); ?>')};" class="listgo-btn listgo-btn--round btn-primary"><?php esc_html_e('Get Coupon', 'wiloke-listgo-widgets'); ?></a>
                </div>
                <?php else: ?>
                <div class="text-center">
                    <a href="javascript:void(0)" onclick="s=prompt('<?php echo esc_attr($aCoupon['description']); ?>','<?php echo esc_attr($aCoupon['coupon_code']); ?>');" class="listgo-btn btn-primary listgo-btn--round"><?php esc_html_e('Get Coupon', 'wiloke-listgo-widgets'); ?></a>
                </div>
                <?php
                    endif;
                }
                ?>

	            <?php if ( isset($aGeneralSettings['website']) && !empty($aGeneralSettings['website']) ) : ?>
                <div class="widget_author__link">
                    <a href="<?php echo esc_url($aGeneralSettings['website']); ?>" target="_blank"><?php esc_html_e('Visit Website', 'wiloke-listgo-widgets'); ?></a>
                </div>
                <?php elseif ( !empty($aUserInfo['user_url']) ) : ?>
                <div class="widget_author__link">
                    <a href="<?php echo esc_url($aUserInfo['user_url']); ?>" target="_blank"><?php esc_html_e('Visit Website', 'wiloke-listgo-widgets'); ?></a>
                </div>
                <?php endif; ?>

            </div>
        <?php
            echo $atts['after_widget'];
        endif;
    }
}
