<?php
use WilokeWidget\Supports\Helpers;

class WilokeMailChimp extends WP_Widget
{
	public $aDef = array('title'=>'Subscribe', 'term_checkbox'=>'');
	public function __construct()
	{
		parent::__construct('wiloke_mailchimp', WILOKE_WIDGET_PREFIX . esc_html__( 'MailChimp', 'wiloke-listgo-widgets'), array('classname'=>'widget_newsletter'));
	}

	public function form($aInstance)
	{
		$aInstance = wp_parse_args($aInstance, $this->aDef);
		Helpers::textField( esc_html__('Title', 'wiloke-listgo-widgets'), $this->get_field_id('title'), $this->get_field_name('title'), $aInstance['title']);
		Helpers::description('Ensure that MailChimp Information has been configured. If you still not do that, please go to Settings -> Wiloke MailChimp');
		Helpers::textareaField( 'Term Checkbox', $this->get_field_id('term_checkbox'), $this->get_field_name('term_checkbox'), $aInstance['term_checkbox'], 'Put the term link in this field. Eg: By creating an account you agree to our &lt;a href="#">Terms and Conditions&lt;/a> and our &lt;a href="#">Privacy Policy&lt;/a>', '<br><a><h1><h2><h3><h4><h5><h6><strong><i>');
	}

	public function update($aNewInstance, $aOldInstance)
	{
		$aInstance = $aOldInstance;
		foreach ( $aNewInstance as $key => $val )
		{
			$aInstance[$key] = strip_tags($val, '<br><a><h1><h2><h3><h4><h5><h6><strong><i>');
		}
		return $aInstance;
	}

	public function widget($atts, $aInstance) {
		$aInstance = wp_parse_args($aInstance, $this->aDef);

		echo $atts['before_widget'];

		if ( !empty($aInstance['title']) ) {
			print $atts['before_title'] . esc_html($aInstance['title']) . $atts['after_title'];
		}

		$aSubscribeSettings = \Wiloke::getOption('wiloke_subscribe_settings');

		if ( empty($aSubscribeSettings) ){
			WilokeAlert::render_alert(__('You have not configured MailChimp yet! Please go to Settings -> Wiloke MailChimp to do that', 'wiloke-listgo-widgets'), 'warning');
		}

		?>
		<div class="mailchimp__content">
			<p><?php echo esc_html($aSubscribeSettings['description']); ?></p>
			<form class="pi_subscribe">
				<input type="email" class="pi-subscribe-email wiloke-subscribe-email form-remove" placeholder="<?php esc_html_e('Enter your email...', 'wiloke-listgo-widgets'); ?>" value="">
				<button type="submit" class="wiloke-submit-subscribe pi-btn pi-subscribe form-remove" data-sendingtext="<?php esc_html_e('Sending', 'wiloke-listgo-widgets'); ?>"><?php esc_html_e('Send', 'wiloke-listgo-widgets'); ?> <i class="icon_mail_alt"></i></button>
                <?php if ( !empty($aInstance['term_checkbox']) ) : ?>
                <label style="margin-top: 10px; display: block">
                    <input class="has_term" type="hidden" value="1" name="has_term">
                    <input class="term_checkbox" type="checkbox" value="1" required name="term_checkbox">
                    <?php Wiloke::wiloke_kses_simple_html($aInstance['term_checkbox']); ?>
                </label>
                <?php endif; ?>
				<p class="message"></p>
			</form>
		</div>
		<?php
		echo $atts['after_widget'];
	}
}