<?php
/*
 * About Us/Me
 * @copyright Wiloke
 * @website https://wiloke.com
 */
use WilokeWidget\Supports\Helpers;
use WilokeListgoFunctionality\Framework\Helpers\Time;

class WilokeOpeningHours extends WP_Widget
{
	public $aDef = array('title' => 'Opening Hours');

	public function __construct(){
		parent::__construct('wiloke_opening_hours', WILOKE_WIDGET_PREFIX . ' (OSP) Opening Hours', array('classname'=>'widget widget_author_calendar', 'description'=>esc_html__('OSP - Only Single Page. This widget is only available for single listing page.', 'wiloke')) );
	}

	public function form($aInstance){
		$aInstance = wp_parse_args($aInstance, $this->aDef);
		Helpers::textField(esc_html__('Title', 'wiloke'), $this->get_field_id('title'), $this->get_field_name('title'), $aInstance['title']);
	}

	public function update($aNewinstance, $aOldinstance) {
		$aInstance = $aOldinstance;
		foreach ( $aNewinstance as $key => $val ){
			$aInstance[$key] = strip_tags($val);
		}

		return $aInstance;
	}

	public function widget($atts, $aInstance){
		global $post, $wiloke;
		$aInstance = wp_parse_args($aInstance, $this->aDef);
		if ( !is_singular('listing') ){
			return false;
		}
		$toggleBusinessHour = \WilokeListgoFunctionality\Framework\Helpers\GetSettings::getPostMeta($post->ID, 'wiloke_toggle_business_hours');
		
		if ( !empty($toggleBusinessHour) && $toggleBusinessHour === 'disable' ){
			return false;
		}

		$aBusinessHours = \WilokeListgoFunctionality\Framework\Helpers\GetSettings::getBusinessHours($post->ID);

		if ( empty($aBusinessHours) ){
			return false;
		}
		$aDays = array(
			esc_html__('Monday', 'wiloke-listgo-widgets'),
			esc_html__('Tuesday', 'wiloke-listgo-widgets'),
			esc_html__('Wednesday', 'wiloke-listgo-widgets'),
			esc_html__('Thursday', 'wiloke-listgo-widgets'),
			esc_html__('Friday', 'wiloke-listgo-widgets'),
			esc_html__('Saturday', 'wiloke-listgo-widgets'),
			esc_html__('Sunday', 'wiloke-listgo-widgets')
		);
        $timeFormat = isset($wiloke->aThemeOptions['time_format']) ? $wiloke->aThemeOptions['time_format'] : 12;
		echo $atts['before_widget'];
		    echo $atts['before_title'] . '<i class="icon_clock_alt"></i>' . ' '  . esc_html($aInstance['title']) . $atts['after_title'];
		?>
		    <div class="widget_author-calendar">
                <ul>
                    <?php foreach ( $aDays as $key => $day ) :  ?>
                        <li>
                            <span class="day"><?php echo esc_html($day); ?></span>
                            <?php if ( isset($aBusinessHours[$key]['closed']) && !empty($aBusinessHours[$key]['closed']) ) : ?>
                                <span class="time time--close"><?php esc_html_e('Closed', 'wiloke'); ?></span>
                            <?php else:
                                $openingTime =  $aBusinessHours[$key]['open_time'];
                                $closedTime =  $aBusinessHours[$key]['close_time'];
                                $extraOpenTime =  $aBusinessHours[$key]['extra_open_time'];
                                $extraCloseTime =  $aBusinessHours[$key]['extra_close_time'];
                                $format = 'H:i';
                                if ( $timeFormat == 12 ) {
                                	$format = 'h:i A';
                                }
                            ?>
                                <span class="time"><?php echo date_i18n($format, strtotime($openingTime)); ?> - <?php echo date_i18n($format, strtotime($closedTime)); ?></span>
                                <?php
                                if ( $extraOpenTime != $extraCloseTime ) :
                                ?>
                                    <br>
                                    <span class="time"><?php echo date_i18n($format, strtotime($extraOpenTime)); ?> - <?php echo date_i18n($format, strtotime($extraCloseTime)); ?></span>
                                <?php endif; ?>
                            <?php endif; ?>
                        </li>
                    <?php endforeach; ?>
                </ul>
            </div>
		<?php
		echo $atts['after_widget'];
	}
}
