 /* 
    @author: HungPham
    @team: wiloke
 */

(function(factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD
        define(['jquery'], factory);
    } else if (typeof exports === 'object') {
        // CommonJS
        factory(require('jquery'));
    } else {
        // Browser globals
        factory(jQuery);
    }
}(function($) {

    var WilokeIconPicker = function(element, options) {
        this.$element   = $(element);
        this.options    = $.extend({}, WilokeIconPicker.defaults, options);
        this.icon       = this.options.icon;
        this.button     = this.options.button;
        this.target     = this.options.target;
        this.search     = this.options.search;
        this.list       = this.options.list;
        this.url    = this.options.url;
        this.data       = this.options.data;
        this.init();
    };

    WilokeIconPicker.defaults  = {
        icon: '.wil-icon-selected',
        button: '.wil-icon-button',
        target: '.wil-icon-value',
        search: '.wil-icon-search-input',
        list: '.wil-icon-list',
        url: '',
        data: ''
    };

    WilokeIconPicker.prototype.init = function() {
        this.onSearch();
        this.onDropdown();
        this.onSelected();
    }

    WilokeIconPicker.prototype.onDropdown = function() {

        var _this = this;

        $(this.button, this.$element).on('click', function(event) {
            event.preventDefault();

            if( !_this.$element.hasClass('is-ajaxed') ) {
                _this.getAjax();
            } else {

                var current = _this.$element.find(_this.target).val();

                if(current =='' ) {
                    _this.$element.find('.selected').removeClass('selected');
                }
            }

            _this.$element.toggleClass('is-dropdown');

        });

        $(document).on('click', function(event) {
            
            if($(event.target).closest(_this.$element).length == 0) {
                _this.$element.removeClass('is-dropdown');
            }

        });
    }

    WilokeIconPicker.prototype.onSearch = function() {

        var $list = $(this.list, this.$element),
            $search = $(this.search, this.$element),
            timeout;

        $search.on('keyup', function(event) {

            var s = $(this).val();

            if(timeout) {
                clearTimeout(timeout);
            }

            if(s.length) {

                timeout = setTimeout(function() {
                    
                    $('li', $list).each(function(index, el) {

                        var $this = $(this),
                            title = $this.data('name');
                        
                        if(title && title.search(s) == -1) {
                            $this.addClass('hidden');
                        } else {
                            $this.removeClass('hidden');
                        }

                    });

                    clearTimeout(timeout);

                }, 100);

            } else {
                $('li', $list).removeClass('hidden');
            }
        });
    }

    WilokeIconPicker.prototype.getAjax = function() {

        var _this = this,
            current = _this.$element.find(_this.target).val();

        if( _this.$element.hasClass('is-ajax') ) {
            return;
        }

        $.ajax({
            url: _this.url,
            type: 'POST',
            data: _this.data,
            beforeSend: function(res) {
                _this.$element.addClass('is-ajax');
                _this.$element.find(_this.list).addClass('loading');
            },
            success: function(res) {

                if(res) {

                    data = JSON.parse(res);

                    if(data.length) {

                        _this.$element.find(_this.list).append('<li data-value=""><i></i></li>');

                        $.each(data, function(index, el) {

                            for (var i in el) {
                                var selected = (current && current == i) ? 'selected' : '';
                                $html = '<li class="'+ selected +'" data-value="'+ i +'" data-name="'+ el[i] +'" title="'+ el[i] +'"><i class="'+ i +'"></i></li>';
                                _this.$element.find(_this.list).append($html);
                            }
                        });
                    }
                }

                _this.$element.removeClass('is-ajax').addClass('is-ajaxed');
                _this.$element.find(_this.list).removeClass('loading');
            }
        });
        
    }

    WilokeIconPicker.prototype.onSelected = function() {

        var _this = this;

        $(_this.list,_this.$element).on('click', 'li', function(event) {
            event.preventDefault();

            var $this = $(this),
                value = $this.data('value');

            if(value) {
                $(_this.target, _this.$element).val(value);
            } else {
                $(_this.target, _this.$element).val('');
            }

            $(_this.search, _this.$element).val('');
            $(_this.list +' li',_this.$element).removeClass('hidden');
            $(_this.icon + ' i', _this.$element).attr('class', value);
            _this.$element.removeClass('is-dropdown');

        });
    }

    $.fn.WilokeIconPicker = function(options) {
        return this.each(function() {
            new WilokeIconPicker(this, options);
        });
    };

}));
