 /* 
    @author: HungPham
    @team: wiloke
 */

(function(factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD
        define(['jquery'], factory);
    } else if (typeof exports === 'object') {
        // CommonJS
        factory(require('jquery'));
    } else {
        // Browser globals
        factory(jQuery);
    }
}(function($) {

    var WilokeWpMedia  = function(element, options) {
        this.$element   = $(element);
        this.options    = $.extend({}, WilokeWpMedia.defaults, options);
        this.buttonAdd  = this.options.buttonAdd;
        this.buttonRemove  = this.options.buttonRemove;
        this.id      = this.options.id;
        this.screen     = this.options.screen;
        this.multiple       = this.options.multiple;
        this.init();
    };

    WilokeWpMedia.defaults  = {
        buttonAdd: '.wil-media-add',
        buttonRemove: '.wil-media-remove',
        id: '.wil-media-id',
        screen: '.wil-media-screen',
        multiple: true
    };

    WilokeWpMedia.prototype.init = function() {
        this.addMedia();
        this.removeAll();
        this.removeItem();
    }

    WilokeWpMedia.prototype.removeAll = function() {
        var _this = this;

        $(_this.buttonRemove, _this.$element).on('click', function(event) {
            event.preventDefault();
            
            $(_this.id, _this.$element).val('');
            $(_this.screen, _this.$element).html('');

        });
    }

    WilokeWpMedia.prototype.removeItem = function() {

        var _this = this;

        $(_this.screen, _this.$element).on('click', 'span', function(event) {
            event.preventDefault();

            var $this = $(this),
                id = $this.data('id'),
                ids = $(_this.id, _this.$element).val();

            ids = ids.split(',');

            ids.splice( ids.indexOf( id.toString()) , 1);

            $this.closest('.wil-media-screen-item').remove();

            $(_this.id, _this.$element).val(ids.join());

        });
    }

    WilokeWpMedia.prototype.addMedia = function() {
        var _this = this,
            frame;

        $(this.buttonAdd, this.$element).on('click', function(event) {
            event.preventDefault();

            if (frame) {
                frame.open();
                return;
            }

            //Extend the wp.media object
            frame = wp.media({
                title: 'Insert Media',
                button: {
                    text: 'Insert'
                },
                multiple: _this.multiple  // Set to true to allow multiple files to be selected
            });

            frame.on('select', function() {
                console.log(frame.state());
                // Get media attachment details from the frame state
                var attachment, html = '', ids = [];

                attachment = frame.state().get('selection').toJSON();

                $.each(attachment, function(index, item) {
                    html += '<li class="wil-media-screen-item" style="background-image: url('+ item.url +')">';
                    html += '<span class="wil-media-screen-item-remove" data-id="'+ item.id +'"></span>';
                    html += '</li>';
                    ids.push(item.id);
                });

                $(_this.id, _this.$element).val(ids.join());
                $(_this.screen, _this.$element).html(html);
                
            });

            //Open the uploader dialog
            frame.open();
        });

    }

    $.fn.WilokeWpMedia = function(options) {
        return this.each(function() {
            new WilokeWpMedia(this, options);
        });
    };

}));
