/* 
   @Author: HungPham
   @Team: wiloke
*/

(function (factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD
        define(['jquery'], factory);
    } else if (typeof exports === 'object') {
        // CommonJS
        factory(require('jquery'));
    } else {
        // Browser globals
        factory(jQuery);
    }
}(function ($) {

    var WilokeMegaMenu = function (el, options) {
        this.$el = $(el);
        this.options = $.extend({}, WilokeMegaMenu.defaults, options, this.dataOption());
        this.alignMenu = this.options.alignMenu;
        this.style = this.options.style;
        this.theme = this.options.theme;
        this.hover = this.options.hover;
        this.breakpoint = this.options.breakpoint;
        this.menuClass = this.options.menuClass;
        this.itemClass = this.options.itemClass;
        this.megaClass = this.options.megaClass;
        this.subClass = this.options.subClass;
        this.dropdownClass = this.options.dropdownClass;
        this.barText = this.options.barText;

        this.init();
    };

    WilokeMegaMenu.defaults = {
        alignMenu: 'left',                              /* options left, right. Default left.*/
        theme: '',
        breakpoint: 767,
        hover: true,
        style: 'wiloke-menu-horizontal',                        /* options wiloke-menu--horizontal, wiloke-menu--vertical. Default horizontal.*/
        menuClass: '.wiloke-menu-list',
        itemClass: '.wiloke-menu-item',
        megaClass: '.wiloke-menu-sub-mega',
        dropdownClass: '.wiloke-menu-sub-dropdown',
        subClass: '.wiloke-menu-sub',
    };

    WilokeMegaMenu.prototype.dataOption = function () {
        return this.$el.data('options');
    }

    WilokeMegaMenu.prototype.init = function () {
        this.onLoad();
        this.onHover();
        this.onClick();
        this.MenuBar();
        this.onResize();
        this.onToggle();
    }

    WilokeMegaMenu.prototype.horizontal = function () {

        var _this = this,
            elWidth = _this.$el.offsetParent().innerWidth(),
            elOffset = _this.$el.offsetParent().offset();

        _this.$el.find(_this.subClass).each(function (index, el) {

            var self = $(this),
                left = 0,
                top = 0,
                right = 'auto',
                width = self.data().width,
                align = self.data().align,
                $parent = self.parent(_this.itemClass),
                parentOffset = $parent.offset();

            if (typeof (width) == 'undefined' || width == '' || isNaN(width)) {
                width = 250;
            }

            if (typeof (align) == 'undefined' || align == '') {
                align = 'right';
            }

            if (width > elWidth) {
                width = elWidth;
            }

            if (align == 'center') {

                if ($parent.hasClass('wiloke-menu-item-level-0')) {

                    var _widthL = (parentOffset.left - elOffset.left) + $parent.innerWidth() / 2,
                        _widthR = elWidth - _widthL;

                    if (_widthL < width / 2) {

                        left = -(parentOffset.left - elOffset.left);

                    } else {

                        if (_widthR < width / 2) {
                            left = -(width / 2 + (width / 2 - _widthR)) + $parent.innerWidth() / 2;
                        } else {
                            left = -width / 2 + $parent.innerWidth() / 2;
                        }
                    }

                } else {
                    align = 'right';
                }
            }

            if (align == 'left') {

                if ($parent.hasClass('wiloke-menu-item-level-0')) {

                    var _width = parentOffset.left - elOffset.left + $parent.innerWidth();

                    if (_width > width) {
                        left = -width + $parent.innerWidth();
                    } else {
                        left = -(parentOffset.left - elOffset.left);
                    }

                } else {

                    var _width = parentOffset.left - elOffset.left;

                    if (_width < width) {
                        width = _width
                    }

                    left = 'auto';
                    right = '100%';
                }
            }

            if (align == 'right') {

                if ($parent.hasClass('wiloke-menu-item-level-0')) {

                    var _width = elWidth - (parentOffset.left - elOffset.left);

                    if (_width > width) {
                        left = 0;
                    } else {
                        left = _width - width;
                    }

                } else {

                    var _width = elWidth - (parentOffset.left - elOffset.left) - $parent.innerWidth();

                    if (_width < width) {
                        width = _width
                    }

                    left = '100%';
                }
            }

            if($parent.hasClass('wiloke-menu-item-level-0')) {
                top = $parent.innerHeight();
            } else {

                if ( self.hasClass(_this.megaClass.replace('.', '')) ) {

                    var _top = $parent.position().top,
                        height = self.innerHeight(),
                        parentSubHeight = $parent.parent(_this.dropdownClass).innerHeight();

                    if (height < parentSubHeight) {

                        var _height = parentSubHeight - _top;

                        if (_height < height) {
                            top = -(height - _height);
                        }

                    } else {
                        top = -_top;
                    }
                }
            }

            if (self.children().length == 0) {
                self.addClass('wiloke-menu--hide');
                self.next('ul').css('background', self.css('background'));
                self.next('ul').find('ul').css('background', self.css('background'));
            }

            self.css({
                'width': width,
                'left': left,
                'top': top,
                'right': right,
                'bottom': 'auto'
            });

        });
    }

    WilokeMegaMenu.prototype.vertical = function () {

        var that = this,
            elOffset = that.$el.offset(),
            elWidth = that.$el.innerWidth(),
            elHeight = that.$el.innerHeight(),
            winWidth = $(window).innerWidth();

        that.$el.find(that.subClass).each(function (index, el) {

            var self = $(this),
                top = 0, 
                left = 0,
                right = 0,
                width = self.data().width,
                parent = self.closest(that.itemClass),
                parentWidth = parent.innerWidth(),
                parentOffset = parent.offset();

            if (typeof width == 'undefined' || width == '' || isNaN(width)) {
                width = 250;
            }

            if (!parent.hasClass('wiloke-menu-item-level-0')) {
                elHeight = parent.parent(that.dropdownClass).innerHeight();
            }

            if (that.alignMenu == 'left') {

                var _width = winWidth - (parentOffset.left + parentWidth + elOffset.left);

                if (_width < width) {
                    width = _width;
                }

                left = '100%';
                right = 'auto';

            } else {

                var _offset = winWidth - (elOffset.left + elWidth),
                    _width = parentOffset.left - _offset;

                if (_width < width) {
                    width = _width;
                }

                left = 'auto';
                right = '100%';
            }

            if (self.hasClass(that.megaClass.replace('.', ''))) {

                var _top = parent.position().top,
                    height = self.innerHeight();

                if (height < elHeight) {

                    var _height = elHeight - _top;

                    if (_height < height) {
                        top = -(height - _height);
                    }

                } else {
                    top = -_top;
                }
            }

            if (self.children().length == 0) {
                self.addClass('wiloke-menu--hide');
                self.next('ul').css('background', self.css('background'));
                self.next('ul').find('ul').css('background', self.css('background'));
            }

            self.css({
                'width': width,
                'top': top,
                'left': left,
                'right': right
            });

        });
    }

    WilokeMegaMenu.prototype.onLoad = function () {

        var winWidth = $(window).innerWidth();

        if (this.breakpoint <= winWidth) {

            this.$el.offsetParent().removeClass('wiloke-menu-responsive');

            this.$el
                .removeClass('wiloke-menu-responsive')
                .find(this.menuClass).css('display', '');

            if (this.style == 'wiloke-menu-vertical') {
                this.vertical();
            } else {
                this.horizontal();
            }

        } else {
            this.$el.addClass('wiloke-menu-responsive');
            this.$el.offsetParent().addClass('wiloke-menu-responsive');
            this.responsive();
        }
    }

    WilokeMegaMenu.prototype.responsive = function () {

        var _this = this;

        _this.$el.find(_this.subClass).each(function (index, el) {

            var self = $(el);

            self.css({
                'top': '',
                'bottom': '',
                'left': '',
                'right': '',
                'width': ''
            });

        });
    }

    WilokeMegaMenu.prototype.onResize = function () {

        var _this = this;

        $(window).on('resize', function (event) {

            var _onResize = setTimeout(function () {
                _this.onLoad();
                clearTimeout(_onResize);
            }, 50);
        });
    }

    WilokeMegaMenu.prototype.onHover = function () {

        var _this = this,
            timeout = '';

        _this.$el.find(_this.itemClass).on({

            mouseenter: function () {

                var win_w = $(window).innerWidth();

                if ( !_this.hover || _this.breakpoint > win_w ) {
                    return;
                }

                var self = $(this),
                    sub = self.find('>' + _this.subClass),
                    effect = sub.data('animated');

                if (sub.length) {

                    self.addClass('wiloke-menu-item-active');

                    sub.css({
                        'visibility': 'visible',
                        'opacity': 1
                    });

                    if (typeof effect != 'undefined' && effect != '') {

                        sub.addClass('animated ' + effect);

                        var clear = setTimeout(function () {
                            sub.removeClass('animated ' + effect);
                            clearTimeout(clear);
                        }, 1000);
                    }
                }
            },
            mouseleave: function () {

                var win_w = $(window).innerWidth();

                if (!_this.hover || _this.breakpoint > win_w) {
                    return;
                }

                var self = $(this),
                    sub = self.find('>' + _this.subClass),
                    effect = sub.data('animated');

                if (sub.length) {
                    self.removeClass('wiloke-menu-item-active');
                    sub.css({
                        'display': '',
                        'visibility': '',
                        'opacity': ''
                    });

                    if (typeof effect != 'undefined' && effect != '') {
                        sub.removeClass('animated ' + effect);
                    }
                }
            }
        });
    }

    WilokeMegaMenu.prototype.onClick = function () {

        var _this = this;

        _this.$el.find(_this.itemClass).on('click', '> a', function (event) {

            var win_w = $(window).innerWidth();

            if (_this.hover || _this.breakpoint > win_w) {
                return;
            }

            var self = $(this),
                li = self.parent(_this.itemClass),
                sub = li.find('> ' + _this.subClass),
                parent = li.parent();

            parent.find('> ' + _this.itemClass).not(li).each(function(index, el) {

                var _li = $(this);
                _li.removeClass('wiloke-menu-item-active');
                _li.find(_this.subClass).css({
                    'display': '',
                    'visibility': '',
                    'opacity': ''
                })
                .removeClass('animated')
                .find(_this.itemClass)
                .removeClass('wiloke-menu-item-active');

            });

            if (sub.length) {

                if ( !li.hasClass('wiloke-menu-item-active') ) {

                    li.addClass('wiloke-menu-item-active');

                    var animated = sub.data('animated');

                    if (typeof animated != 'undefined' && animated != '') {

                        sub.addClass('animated ' + animated);

                        var clear = setTimeout(function () {
                            sub.removeClass('animated ' + animated);
                            clearTimeout(clear);
                        }, 1000);
                    }

                    sub.css({
                        'display': 'block',
                        'visibility': 'visible',
                        'opacity': 1
                    });

                } else {

                    var animated = sub.data('animated');

                    li.removeClass('wiloke-menu-item-active').find(_this.subClass).css({
                        'display': '',
                        'visibility': '',
                        'opacity': ''
                    });

                    sub.removeClass('animated ' + animated);
                }

                return false;
            }

        });

        $(document).on('click', function(event) {

            var self = $(event.target);

            if( !self.closest(_this.$el[0]).length ) {
                
                _this.$el
                .find(_this.itemClass).removeClass('wiloke-menu-item-active')
                .find(_this.subClass).each(function(index, el) {
                    var _self = $(this),
                        effect = _self.data('animated');
                    if(effect) {
                        _self.removeClass('animated ' + effect)
                    }

                    _self.css({
                        'display': '',
                        'visibility': '',
                        'opacity': ''
                    });
                });
            }
        });
    }

    WilokeMegaMenu.prototype.onToggle = function() {

        var _this = this;

        $(_this.$el).on('click', _this.itemClass + ' > a' ,function(event) {

            var winWidth = $(window).innerWidth();

            if(_this.breakpoint > winWidth) {

                var self = $(this),
                    $parent = self.parent( _this.itemClass ),
                    $submenu = $parent.find('> ' + _this.subClass),
                    $ul  = $parent.parent();

                if($ul.length) {

                    $ul.find('> ' + _this.itemClass).not($parent).each(function(index, el) {

                        var _self = $(this),
                            $submenu = _self.find('> ' + _this.subClass);

                        if( $submenu.length ) {

                            _self.removeClass('wiloke-menu-item-active');

                            $submenu.slideUp(300, function() {

                                $submenu.css({
                                    'opacity': '',
                                    'visibility': '',
                                    'display': ''
                                });

                            });
                        }
                    });
                }

                if($submenu.length) {

                    $parent.toggleClass('wiloke-menu-item-active');

                    $submenu.slideToggle(300, function() {

                        if( $parent.hasClass('wiloke-menu-item-active') ) {

                            $submenu.css({
                                'opacity': 1,
                                'visibility': 'visible'
                            });

                        } else {

                            $submenu.css({
                                'opacity': '',
                                'visibility': '',
                                'display': ''
                            });

                        }
                    });

                    
                    return false;
                }
            }

        });
    }

    WilokeMegaMenu.prototype.MenuBar = function() {

        var _this = this;
        
        var $menubar = $('<div class="wiloke-menu-bar"><span class="wiloke-menu-bar__line"><span></span><span></span><span></span></span></div>');

        if(this.barText !='') {
            $menubar.append('<span class="wiloke-menu-bar__text">'+ this.barText +'</span>');
        }

        _this.$el.prepend($menubar);

        $menubar.on('click', function() {

            var winWidth = $(window).innerWidth();

            if(_this.breakpoint > winWidth) {
                $menubar.toggleClass('wiloke-menu-bar-active');
                _this.$el.find(_this.menuClass).slideToggle();
            }

        });

    }

    $.fn.WilokeMegaMenu = function (options) {

        return this.each(function () {

            var $this = $(this),
                data = $this.data('WilokeMegaMenu');

            if (!data) {
                data = new WilokeMegaMenu(this, options);
                $this.data('WilokeMegaMenu', data);
            }

        });
    };

}));
